@@ -122,8 +122,10 @@ void asr_mod(const std::string &src) {
 
     std::string modfile = LCompilers::save_modfile(*asr, lm);
     LCompilers::SymbolTable symtab(nullptr);
-    LCompilers::ASR::TranslationUnit_t *asr2 = LCompilers::load_modfile(al,
-            modfile, true, symtab, lm);
+    LCompilers::Result<LCompilers::ASR::TranslationUnit_t*, LCompilers::ErrorMessage> res
+        = LCompilers::load_modfile(al, modfile, true, symtab, lm);
+    CHECK(res.ok);
+    LCompilers::ASR::TranslationUnit_t* asr2 = res.result;
     fix_external_symbols(*asr2, symtab);
     LCOMPILERS_ASSERT(LCompilers::asr_verify(*asr2, true, diagnostics));
 
