@@ -36,22 +36,22 @@ class ASRBuilder {
 
     // Symbols -----------------------------------------------------------------
     ASR::expr_t *Variable(SymbolTable *symtab, std::string var_name,
-            ASR::ttype_t *type, ASR::intentType intent,
+            ASR::ttype_t *type, ASR::intentType intent, ASR::symbol_t* type_decl=nullptr,
             ASR::abiType abi=ASR::abiType::Source, bool a_value_attr=false) {
         ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(
             ASRUtils::make_Variable_t_util(al, loc, symtab, s2c(al, var_name), nullptr, 0,
-            intent, nullptr, nullptr, ASR::storage_typeType::Default, type, nullptr, abi,
+            intent, nullptr, nullptr, ASR::storage_typeType::Default, type, type_decl, abi,
             ASR::Public, ASR::presenceType::Required, a_value_attr));
         symtab->add_symbol(s2c(al, var_name), sym);
         return ASRUtils::EXPR(ASR::make_Var_t(al, loc, sym));
     }
 
     void VariableDeclaration(SymbolTable *symtab, std::string var_name,
-            ASR::ttype_t *type, ASR::intentType intent,
+            ASR::ttype_t *type, ASR::intentType intent, ASR::symbol_t* type_decl=nullptr,
             ASR::abiType abi=ASR::abiType::Source, bool a_value_attr=false) {
         ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(
             ASRUtils::make_Variable_t_util(al, loc, symtab, s2c(al, var_name), nullptr, 0,
-            intent, nullptr, nullptr, ASR::storage_typeType::Default, type, nullptr, abi,
+            intent, nullptr, nullptr, ASR::storage_typeType::Default, type, type_decl, abi,
             ASR::Public, ASR::presenceType::Required, a_value_attr));
         symtab->add_symbol(s2c(al, var_name), sym);
         return;
@@ -70,15 +70,27 @@ class ASRBuilder {
 
     #define declare(var_name, type, intent)                                     \
         b.Variable(fn_symtab, var_name, type, ASR::intentType::intent)
+    
+    #define declare_struct_type(var_name, type, intent, m_arg)                   \
+        b.Variable(fn_symtab, var_name, type, ASR::intentType::intent,           \
+            ASRUtils::get_struct_sym_from_struct_expr(m_arg))
 
     #define fill_func_arg(arg_name, type) {                                     \
         auto arg = declare(arg_name, type, In);                                 \
         args.push_back(al, arg); }
 
+    #define fill_func_arg_struct_type(arg_name, type, m_arg) {                         \
+        auto arg = declare_struct_type(arg_name, type, In, m_arg);                                 \
+        args.push_back(al, arg); }
+
     #define fill_func_arg_sub(arg_name, type, intent) {                                     \
         auto arg = declare(arg_name, type, intent);                                 \
         args.push_back(al, arg); }
 
+    #define fill_func_arg_sub_struct_type(arg_name, type, intent, m_arg) {                                     \
+        auto arg = declare_struct_type(arg_name, type, intent, m_arg);                                 \
+        args.push_back(al, arg); }
+
     #define make_ASR_Function_t(name, symtab, dep, args, body, return_var, abi,     \
             deftype, bindc_name)                                                \
         ASR::down_cast<ASR::symbol_t>( ASRUtils::make_Function_t_util(al, loc,  \
@@ -105,7 +117,15 @@ class ASRBuilder {
     #define complex32    ASRUtils::TYPE(ASR::make_Complex_t(al, loc, 4))
     #define complex64    ASRUtils::TYPE(ASR::make_Complex_t(al, loc, 8))
     #define logical      ASRUtils::TYPE(ASR::make_Logical_t(al, loc, 4))
-    #define character(x) ASRUtils::TYPE(ASR::make_String_t(al, loc, 1, x, nullptr, ASR::string_physical_typeType::PointerString))
+    #define character(x) ASRUtils::TYPE(ASR::make_String_t(al, loc, 1,\
+                            ASRUtils::EXPR(ASR::make_IntegerConstant_t(\
+                            al, loc, x, int32)),\
+                            ASR::string_length_kindType::ExpressionLength,\
+                            ASR::string_physical_typeType::DescriptorString))
+    #define allocatable_deferred_string() ASRUtils::TYPE(ASR::make_Allocatable_t(al, loc,\
+                                ASRUtils::TYPE(ASR::make_String_t(al, loc, 1, nullptr,\
+                            ASR::string_length_kindType::DeferredLength,\
+                            ASR::string_physical_typeType::DescriptorString))))
     #define List(x)      ASRUtils::TYPE(ASR::make_List_t(al, loc, x))
 
     ASR::ttype_t *Tuple(std::vector<ASR::ttype_t*> tuple_type) {
@@ -133,6 +153,20 @@ class ASRBuilder {
         return make_Array_t_util(al, loc, type, m_dims.p, m_dims.n);
     }
 
+    ASR::ttype_t* UnboundedArray(ASR::ttype_t* type, int32_t n_dims){
+        Vec<ASR::dimension_t> arr_dimensions; arr_dimensions.reserve(al, 1);
+        for(int i = 0; i < n_dims; i++) {
+            arr_dimensions.push_back(al, ASR::dimension_t{loc, nullptr, nullptr});
+        }
+        ASR::ttype_t* array_type = ASRUtils::TYPE(
+            ASR::make_Array_t(
+                al, loc,
+                type,
+                arr_dimensions.p, arr_dimensions.n,
+                ASR::UnboundedPointerArray));
+        return array_type;
+    }
+
     ASR::ttype_t* CPtr() {
         return TYPE(ASR::make_CPtr_t(al, loc));
     }
@@ -142,6 +176,27 @@ class ASRBuilder {
         return ASRUtils::EXPR(ASR::make_Var_t(al, loc, sym));
     }
 
+    ASR::ttype_t* String(ASR::expr_t* len,
+        ASR::string_length_kindType len_kind, 
+        ASR::string_physical_typeType physical_type = ASR::DescriptorString) {
+        if(!(
+                (len_kind == ASR::AssumedLength && !len) || 
+                (len_kind == ASR::DeferredLength && !len) ||
+                (len_kind == ASR::ExpressionLength && len)
+            )){
+            throw LCompilersException("Invalid String Node Status");
+        }
+
+        if(physical_type == ASR::CChar){
+            int64_t len_const;
+            if(!is_value_constant(len, len_const))
+                LCompilersException("Invalid String Node Status");
+        }
+
+        return ASRUtils::TYPE(ASR::make_String_t(al, loc, 1, len,
+            len_kind, physical_type));
+    }
+
     ASR::expr_t* ArrayUBound(ASR::expr_t* x, int64_t dim) {
         ASR::expr_t* value = nullptr;
         ASR::ttype_t* type = ASRUtils::expr_type(x);
@@ -275,11 +330,21 @@ class ASRBuilder {
     }
 
     inline ASR::expr_t* StringSection(ASR::expr_t* s, ASR::expr_t* start, ASR::expr_t* end) {
-        return EXPR(ASR::make_StringSection_t(al, loc, s, start, end, i32(1), character(-2), nullptr));
+        int64_t start_const, end_const;
+        ASR::ttype_t* string_type {};
+        LCOMPILERS_ASSERT(start && end)
+        if( ASRUtils::is_value_constant(start, start_const) &&
+            ASRUtils::is_value_constant(end, end_const)){
+            string_type = character(end_const - start_const + 1);
+        } else {
+            string_type = String(Add(Sub(end, start),i_t(1, expr_type(start))),
+                ASR::string_length_kindType::ExpressionLength);
+        }
+        return EXPR(ASR::make_StringSection_t(al, loc, s, start, end, i32(1), string_type, nullptr));
     }
 
     inline ASR::expr_t* StringItem(ASR::expr_t* x, ASR::expr_t* idx) {
-        return EXPR(ASR::make_StringItem_t(al, loc, x, idx, character(-2), nullptr));
+        return EXPR(ASR::make_StringItem_t(al, loc, x, idx, character(1), nullptr));
     }
 
     inline ASR::expr_t* StringConstant(std::string s, ASR::ttype_t* type) {
@@ -295,7 +360,7 @@ class ASRBuilder {
     }
 
     inline ASR::expr_t* ArraySize(ASR::expr_t* x, ASR::expr_t* dim, ASR::ttype_t* t) {
-        return EXPR(make_ArraySize_t_util(al, loc, x, dim, t, nullptr));
+        return EXPR(make_ArraySize_t_util(al, loc, x, dim, t, nullptr, false));
     }
 
     inline ASR::expr_t* Ichar(std::string s, ASR::ttype_t* type, ASR::ttype_t* t) {
@@ -362,8 +427,12 @@ class ASRBuilder {
     }
 
     inline ASR::expr_t* c2r_t(ASR::expr_t* x, ASR::ttype_t* t) {
-        // TODO: handle value
-        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::ComplexToReal, t, nullptr));
+        ASR::expr_t* value = ASRUtils::expr_value(x);
+        if ( value != nullptr ) {
+            double re = ASR::down_cast<ASR::ComplexConstant_t>(value)->m_re;
+            value = f_t(re, t);
+        }
+        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::ComplexToReal, t, value));
     }
 
     inline ASR::expr_t* t2t(ASR::expr_t* x, ASR::ttype_t* t1, ASR::ttype_t* t2) {
@@ -407,9 +476,17 @@ class ASRBuilder {
         return EXPR(ASR::make_IntegerBinOp_t(al, loc, n, ASR::binopType::BitLShift, bits, t, nullptr));
     }
 
+    inline ASR::expr_t* LBitRshift(ASR::expr_t* n, ASR::expr_t* bits, ASR::ttype_t* t) {
+        return EXPR(ASR::make_IntegerBinOp_t(al, loc, n, ASR::binopType::LBitRShift, bits, t, nullptr));
+    }
+
+    ASR::expr_t* BitCast(ASR::expr_t* src, ASR::expr_t* mold /*Dest*/, ASR::expr_t* size = nullptr){
+        return EXPR(ASR::make_BitCast_t(al, loc, src, mold, size, ASRUtils::duplicate_type(al, expr_type(mold)), nullptr));
+    }
+
     ASR::expr_t *And(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
-        ASR::ttype_t *type = expr_type(left);
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
+        ASR::ttype_t *type = type_get_past_allocatable_pointer(expr_type(left));
         ASRUtils::make_ArrayBroadcast_t_util(al, loc, left, right);
         switch (type->type) {
             case ASR::ttypeType::Integer: {
@@ -420,15 +497,15 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *Or(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
-        ASR::ttype_t *type = expr_type(left);
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
+        ASR::ttype_t *type = type_get_past_allocatable_pointer(expr_type(left));
         ASRUtils::make_ArrayBroadcast_t_util(al, loc, left, right);
         switch (type->type) {
             case ASR::ttypeType::Integer: {
@@ -439,15 +516,15 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *Xor(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
-        ASR::ttype_t *type = expr_type(left);
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
+        ASR::ttype_t *type = type_get_past_allocatable_pointer(expr_type(left));
         ASRUtils::make_ArrayBroadcast_t_util(al, loc, left, right);
         switch (type->type) {
             case ASR::ttypeType::Integer: {
@@ -458,14 +535,14 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *Not(ASR::expr_t *x) {
-        ASR::ttype_t *type = expr_type(x);
+        ASR::ttype_t *type = type_get_past_allocatable_pointer(expr_type(x));
         switch (type->type) {
             case ASR::ttypeType::Integer: {
                 return EXPR(ASR::make_IntegerBitNot_t(al, loc, x, type, nullptr));
@@ -475,15 +552,15 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(x)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(x), x) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *Add(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
-        ASR::ttype_t *type = expr_type(left);
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
+        ASR::ttype_t *type = type_get_past_allocatable_pointer(expr_type(left));
         ASRUtils::make_ArrayBroadcast_t_util(al, loc, left, right);
         switch (type->type) {
             case ASR::ttypeType::Integer : {
@@ -504,15 +581,15 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *Sub(ASR::expr_t *left, ASR::expr_t *right, ASR::expr_t* value = nullptr) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
-        ASR::ttype_t *type = expr_type(left);
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
+        ASR::ttype_t *type = type_get_past_allocatable_pointer(expr_type(left));
         ASRUtils::make_ArrayBroadcast_t_util(al, loc, left, right);
         switch (type->type) {
             case ASR::ttypeType::Integer: {
@@ -529,15 +606,15 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *Mul(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
-        ASR::ttype_t *type = expr_type(left);
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
+        ASR::ttype_t *type = type_get_past_allocatable_pointer(expr_type(left));
         ASRUtils::make_ArrayBroadcast_t_util(al, loc, left, right);
         switch (type->type) {
             case ASR::ttypeType::Integer: {
@@ -568,15 +645,15 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *Div(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
-        ASR::ttype_t *type = expr_type(left);
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
+        ASR::ttype_t *type = type_get_past_allocatable_pointer(expr_type(left));
         ASRUtils::make_ArrayBroadcast_t_util(al, loc, left, right);
         switch (type->type) {
             case ASR::ttypeType::Integer: {
@@ -593,15 +670,15 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *Pow(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
-        ASR::ttype_t *type = expr_type(left);
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
+        ASR::ttype_t *type = type_get_past_allocatable_pointer(expr_type(left));
         ASRUtils::make_ArrayBroadcast_t_util(al, loc, left, right);
         switch (type->type) {
             case ASR::ttypeType::Integer: {
@@ -618,7 +695,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
@@ -666,7 +743,7 @@ class ASRBuilder {
 
     // Compare -----------------------------------------------------------------
     ASR::expr_t *Gt(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
         ASR::ttype_t *type = expr_type(left);
         switch(type->type){
             case ASR::ttypeType::Integer: {
@@ -683,14 +760,14 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *Lt(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right),left, right));
         ASR::ttype_t *type = expr_type(left);
         switch(type->type){
             case ASR::ttypeType::Integer: {
@@ -707,14 +784,14 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *GtE(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
         ASR::ttype_t *type = expr_type(left);
         switch(type->type){
             case ASR::ttypeType::Integer: {
@@ -731,14 +808,14 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *LtE(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
         ASR::ttype_t *type = expr_type(left);
         switch(type->type){
             case ASR::ttypeType::Integer: {
@@ -755,14 +832,14 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *Eq(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
         ASR::ttype_t *type = expr_type(left);
         switch(type->type){
             case ASR::ttypeType::Integer: {
@@ -782,14 +859,14 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
     }
 
     ASR::expr_t *NotEq(ASR::expr_t *left, ASR::expr_t *right) {
-        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right)));
+        LCOMPILERS_ASSERT(check_equal_type(expr_type(left), expr_type(right), left, right));
         ASR::ttype_t *type = expr_type(left);
         switch(type->type){
             case ASR::ttypeType::Integer: {
@@ -806,7 +883,7 @@ class ASRBuilder {
             }
             default: {
                 throw LCompilersException("Expression type, " +
-                    ASRUtils::type_to_str_python(expr_type(left)) + " not yet supported");
+                    ASRUtils::type_to_str_python_expr(expr_type(left), left) + " not yet supported");
                 return nullptr;
             }
         }
@@ -820,7 +897,7 @@ class ASRBuilder {
         Vec<ASR::stmt_t*> m_else_body; m_else_body.reserve(al, 1);
         for (auto &x: else_body) m_else_body.push_back(al, x);
 
-        return STMT(ASR::make_If_t(al, loc, a_test, m_if_body.p, m_if_body.n,
+        return STMT(ASR::make_If_t(al, loc, nullptr, a_test, m_if_body.p, m_if_body.n,
             m_else_body.p, m_else_body.n));
     }
 
@@ -841,8 +918,8 @@ class ASRBuilder {
     ASR::expr_t* Call(ASR::symbol_t* s, Vec<ASR::call_arg_t>& args,
                       ASR::ttype_t* return_type) {
         return ASRUtils::EXPR(ASRUtils::make_FunctionCall_t_util(al, loc,
-                s, s, args.p, args.size(), return_type, nullptr, nullptr,
-                false));
+                s, s, args.p, args.size(), return_type, nullptr, nullptr
+                ));
     }
 
     ASR::expr_t* Call(ASR::symbol_t* s, Vec<ASR::expr_t *>& args,
@@ -850,23 +927,24 @@ class ASRBuilder {
         Vec<ASR::call_arg_t> args_; args_.reserve(al, 2);
         visit_expr_list(al, args, args_);
         return ASRUtils::EXPR(ASRUtils::make_FunctionCall_t_util(al, loc,
-                s, s, args_.p, args_.size(), return_type, nullptr, nullptr,
-                false));
+                s, s, args_.p, args_.size(), return_type, nullptr, nullptr
+                ));
     }
 
     ASR::expr_t* Call(ASR::symbol_t* s, Vec<ASR::call_arg_t>& args,
                       ASR::ttype_t* return_type, ASR::expr_t* value) {
         return ASRUtils::EXPR(ASRUtils::make_FunctionCall_t_util(al, loc,
-                s, s, args.p, args.size(), return_type, value, nullptr,
-                false));
+                s, s, args.p, args.size(), return_type, value, nullptr));
     }
 
     ASR::stmt_t* SubroutineCall(ASR::symbol_t* s, Vec<ASR::call_arg_t>& args) {
         return ASRUtils::STMT(ASRUtils::make_SubroutineCall_t_util(al, loc,
-                s, s, args.p, args.size(), nullptr, nullptr, false, false));
+                s, s, args.p, args.size(), nullptr, nullptr, false));
     }
 
     ASR::expr_t *ArrayItem_01(ASR::expr_t *arr, std::vector<ASR::expr_t*> idx) {
+        // LCOMPILERS_ASSERT(
+        //     ASRUtils::extract_n_dims_from_ttype(ASRUtils::expr_type(arr)) == idx.size());
         Vec<ASR::expr_t*> idx_vars; idx_vars.reserve(al, 1);
         for (auto &x: idx) idx_vars.push_back(al, x);
         return PassUtils::create_array_ref(arr, idx_vars, al);
@@ -911,9 +989,9 @@ class ASRBuilder {
     }
 
     ASR::stmt_t *Assignment(ASR::expr_t *lhs, ASR::expr_t *rhs) {
-        LCOMPILERS_ASSERT_MSG(check_equal_type(expr_type(lhs), expr_type(rhs)),
-            type_to_str_python(expr_type(lhs)) + ", " + type_to_str_python(expr_type(rhs)));
-        return STMT(ASR::make_Assignment_t(al, loc, lhs, rhs, nullptr));
+        LCOMPILERS_ASSERT_MSG(check_equal_type(expr_type(lhs), expr_type(rhs), lhs, rhs),
+            type_to_str_python_expr(expr_type(lhs), lhs) + ", " + type_to_str_python_expr(expr_type(rhs), rhs));
+        return STMT(ASRUtils::make_Assignment_t_util(al, loc, lhs, rhs, nullptr, false, false));
     }
 
     ASR::stmt_t* CPtrToPointer(ASR::expr_t* cptr, ASR::expr_t* ptr, ASR::expr_t* shape = nullptr, ASR::expr_t* lower_bounds = nullptr) {
@@ -949,12 +1027,13 @@ class ASRBuilder {
         alloc_arg.n_dims = dims.n;
         alloc_arg.m_type = nullptr;
         alloc_arg.m_len_expr = nullptr;
+        alloc_arg.m_sym_subclass = nullptr;
         alloc_args.push_back(al, alloc_arg);
         return STMT(ASR::make_Allocate_t(al, loc, alloc_args.p, 1,
             nullptr, nullptr, nullptr));
     }
 
-    ASR::stmt_t *Allocate(ASR::expr_t *m_a, ASR::dimension_t* m_dims, size_t n_dims) {
+    ASR::stmt_t *Allocate(ASR::expr_t *m_a, ASR::dimension_t* m_dims, size_t n_dims, ASR::expr_t* len_expr = nullptr) {
         Vec<ASR::alloc_arg_t> alloc_args; alloc_args.reserve(al, 1);
         ASR::alloc_arg_t alloc_arg;
         alloc_arg.loc = loc;
@@ -962,12 +1041,23 @@ class ASRBuilder {
         alloc_arg.m_dims = m_dims;
         alloc_arg.n_dims = n_dims;
         alloc_arg.m_type = nullptr;
-        alloc_arg.m_len_expr = nullptr;
+        alloc_arg.m_sym_subclass = nullptr;
+        alloc_arg.m_len_expr = len_expr;
         alloc_args.push_back(al, alloc_arg);
         return STMT(ASR::make_Allocate_t(al, loc, alloc_args.p, 1,
             nullptr, nullptr, nullptr));
     }
-
+    ASR::ttype_t* allocatable(ASR::ttype_t* type){
+        return ASRUtils::TYPE(ASR::make_Allocatable_t(al, loc, type));
+    }
+    ASR::stmt_t* Deallocate(ASR::expr_t* m_a){
+        Vec<ASR::expr_t*> m_a_vec; m_a_vec.reserve(al, 1);
+        m_a_vec.push_back(al, m_a);
+        return Deallocate(m_a_vec);
+    }
+    ASR::stmt_t* Deallocate(Vec<ASR::expr_t*> &m_a){
+        return ASRUtils::STMT(ASR::make_ExplicitDeallocate_t(al, loc, m_a.p, m_a.n));
+    }
 
     #define UBound(arr, dim) PassUtils::get_bound(arr, dim, "ubound", al)
     #define LBound(arr, dim) PassUtils::get_bound(arr, dim, "lbound", al)
@@ -1109,7 +1199,7 @@ class ASRBuilder {
             Vec<ASR::stmt_t*> if_body;
             if_body.reserve(al, 1);
             if_body.push_back(al, doloop);
-            ASR::stmt_t* if_ = ASRUtils::STMT(ASR::make_If_t(al, loc, test_expr,
+            ASR::stmt_t* if_ = ASRUtils::STMT(ASR::make_If_t(al, loc, nullptr, test_expr,
                                 if_body.p, if_body.size(), else_, else_n));
             Vec<ASR::stmt_t*> if_else_if;
             if_else_if.reserve(al, 1);
@@ -1132,10 +1222,10 @@ class ASRBuilder {
         Vec<ASR::expr_t*> args_1; args_1.reserve(al, n_args);
         for (int i = 0; i < n_args; i++) {
             args_1.push_back(al, this->Variable(fn_symtab_1, "x_"+std::to_string(i), arg_types[i],
-                ASR::intentType::In, ASR::abiType::BindC, true));
+                ASR::intentType::In, nullptr, ASR::abiType::BindC, true));
         }
         ASR::expr_t *return_var_1 = this->Variable(fn_symtab_1, c_func_name,
-            return_type, ASRUtils::intent_return_var, ASR::abiType::BindC, false);
+            return_type, ASRUtils::intent_return_var, nullptr, ASR::abiType::BindC, false);
 
         SetChar dep_1; dep_1.reserve(al, 1);
         Vec<ASR::stmt_t*> body_1; body_1.reserve(al, 1);
@@ -1149,11 +1239,11 @@ class ASRBuilder {
         Vec<ASR::expr_t*> args_1; args_1.reserve(al, 0);
         for (int i = 0; i < n_args; i++) {
             args_1.push_back(al, this->Variable(fn_symtab_1, "x_"+std::to_string(i), arg_types,
-                ASR::intentType::InOut, ASR::abiType::BindC, true));
+                ASR::intentType::InOut, nullptr, ASR::abiType::BindC, true));
         }
         ASR::expr_t *return_var_1 = this->Variable(fn_symtab_1, c_func_name,
            ASRUtils::type_get_past_array(ASRUtils::type_get_past_allocatable(arg_types)),
-           ASRUtils::intent_return_var, ASR::abiType::BindC, false);
+           ASRUtils::intent_return_var, nullptr, ASR::abiType::BindC, false);
         SetChar dep_1; dep_1.reserve(al, 1);
         Vec<ASR::stmt_t*> body_1; body_1.reserve(al, 1);
         ASR::symbol_t *s = make_ASR_Function_t(c_func_name, fn_symtab_1, dep_1, args_1,
@@ -1161,6 +1251,50 @@ class ASRBuilder {
         return s;
     }
 
+    ASR::symbol_t* create_c_func_subroutines_with_return_type(std::string c_func_name, SymbolTable* fn_symtab, int n_args, std::vector<ASR::ttype_t*> arg_types,
+            ASR::ttype_t* return_type) {
+        SymbolTable *fn_symtab_1 = al.make_new<SymbolTable>(fn_symtab);
+        Vec<ASR::expr_t*> args_1; args_1.reserve(al, 0);
+        LCOMPILERS_ASSERT( (size_t)n_args == arg_types.size())
+        for (int i = 0; i < n_args; i++) {
+            args_1.push_back(al, this->Variable(fn_symtab_1, "x_"+std::to_string(i), arg_types[i],
+                ASR::intentType::InOut, nullptr, ASR::abiType::BindC, true));
+        }
+        ASR::expr_t *return_var_1 = this->Variable(fn_symtab_1, c_func_name,
+           return_type,
+           ASRUtils::intent_return_var, nullptr, ASR::abiType::BindC, false);
+        SetChar dep_1; dep_1.reserve(al, 1);
+        Vec<ASR::stmt_t*> body_1; body_1.reserve(al, 1);
+        ASR::symbol_t *s = make_ASR_Function_t(c_func_name, fn_symtab_1, dep_1, args_1,
+            body_1, return_var_1, ASR::abiType::BindC, ASR::deftypeType::Interface, s2c(al, c_func_name));
+        return s;
+    }
+    /*
+        Creates c-bind subroutine interface
+        e.g. -> `subroutine(arg1, arg2) bind(c)`
+    */
+    ASR::symbol_t* create_c_subroutine_interface(std::string &c_func_name, SymbolTable* fn_symtab,
+        Vec<ASR::ttype_t*> &parameter_types, const std::vector<std::string> &parameter_names, 
+        std::vector<bool> is_parameter_value = {}/*Does parameter have `value` attribute*/){
+        LCOMPILERS_ASSERT(parameter_types.size() == parameter_names.size())
+        LCOMPILERS_ASSERT( is_parameter_value.empty() ||
+            (parameter_types.size() == is_parameter_value.size()))
+
+        SymbolTable* symTable = al.make_new<SymbolTable>(fn_symtab /*parent*/);
+        Vec<ASR::expr_t*> parameters; parameters.reserve(al, parameter_types.size());
+        for (size_t i = 0; i < parameter_types.size(); i++) {
+            parameters.push_back(al, Variable(symTable,
+                parameter_names[i] + std::to_string(i), parameter_types[i],
+                ASR::intentType::InOut, nullptr, ASR::abiType::BindC,
+                is_parameter_value.empty() ? false : is_parameter_value[i]));
+        }
+        SetChar dep; dep.reserve(al, 1);
+        Vec<ASR::stmt_t*> body; body.reserve(al, 1);
+        ASR::symbol_t *s = make_ASR_Function_t(c_func_name, symTable, dep, parameters,
+            body, nullptr, ASR::abiType::BindC, ASR::deftypeType::Interface, s2c(al, c_func_name));
+        return s;
+    }
+
 };
 
 } // namespace LCompilers::ASRUtils
