@@ -40,6 +40,7 @@ private:
     SymbolTable *current_symtab;
     bool check_external;
     diag::Diagnostics &diagnostics;
+    std::string current_name;
 
     // For checking that all symtabs have a unique ID.
     // We first walk all symtabs, and then we check that everything else
@@ -52,13 +53,17 @@ private:
 
     std::set<std::pair<uint64_t, std::string>> const_assigned;
 
-    bool symbol_visited;
+    // checks whether we've visited any `Var`, which isn't a global `Variable`
+    bool non_global_symbol_visited;
     bool _return_var_or_intent_out = false;
     bool _processing_dims = false;
+    bool _inside_call = false;
+    bool _inside_array_physical_cast_type = false;
+    const ASR::expr_t* current_expr {}; // current expression being visited 
 
 public:
     VerifyVisitor(bool check_external, diag::Diagnostics &diagnostics) : check_external{check_external},
-        diagnostics{diagnostics}, symbol_visited{false} {}
+        diagnostics{diagnostics}, non_global_symbol_visited{false} {}
 
     // Requires the condition `cond` to be true. Raise an exception otherwise.
     #define require(cond, error_msg) ASRUtils::require_impl((cond), (error_msg), x.base.base.loc, diagnostics);
@@ -377,33 +382,62 @@ public:
                 std::string variable_name = ASRUtils::symbol_name(target_Var->m_v);
                 require(const_assigned.find(std::make_pair(current_symtab->counter,
                     variable_name)) == const_assigned.end(),
-                    "Assignment target with " + ASRUtils::type_to_str_python(target_type)
+                    "Assignment target with " + ASRUtils::type_to_str_python_expr(target_type, target)
                     + " cannot be re-assigned.");
                 const_assigned.insert(std::make_pair(current_symtab->counter, variable_name));
             }
         }
+        // it's possible that the target is an external symbol, and during
+        // initial deserialization pass, so we don't do the below verification
+        if ( check_external && x.m_realloc_lhs ) {
+            ASR::expr_t* a_target = x.m_target;
+            bool is_allocatable = ASRUtils::is_allocatable(a_target);
+            if ( ASR::is_a<ASR::StructInstanceMember_t>(*a_target) ) {
+                ASR::StructInstanceMember_t* a_target_struct = ASR::down_cast<ASR::StructInstanceMember_t>(a_target);
+                is_allocatable |= ASRUtils::is_allocatable(a_target_struct->m_v);
+            }
+            require(is_allocatable,
+                "Reallocation of non allocatable variable is not allowed");
+        }
+        if (x.m_move_allocation) {
+            ASR::ttype_t* target_type = ASRUtils::expr_type(x.m_target);
+            ASR::ttype_t* value_type = ASRUtils::expr_type(x.m_value);
+
+            bool is_target_allocatable_array = ASRUtils::is_array(target_type) &&
+                                            ASRUtils::is_allocatable(target_type) &&
+                                            ASRUtils::extract_physical_type(target_type) == ASR::array_physical_typeType::DescriptorArray;
+
+            bool is_value_allocatable_array = ASRUtils::is_array(value_type) &&
+                                            ASRUtils::is_allocatable(value_type) &&
+                                            ASRUtils::extract_physical_type(value_type) == ASR::array_physical_typeType::DescriptorArray;
+
+            require(is_target_allocatable_array,
+                "Move assignment target must be an allocatable array");
+            require(is_value_allocatable_array,
+                "Move assignment value must be an allocatable array");
+        }
         BaseWalkVisitor<VerifyVisitor>::visit_Assignment(x);
     }
 
-    void visit_ClassProcedure(const ClassProcedure_t &x) {
+    void visit_StructMethodDeclaration(const StructMethodDeclaration_t &x) {
         require(x.m_name != nullptr,
-            "The ClassProcedure::m_name cannot be nullptr");
+            "The StructMethodDeclaration::m_name cannot be nullptr");
         require(x.m_proc != nullptr,
-            "The ClassProcedure::m_proc cannot be nullptr");
+            "The StructMethodDeclaration::m_proc cannot be nullptr");
         require(x.m_proc_name != nullptr,
-            "The ClassProcedure::m_proc_name cannot be nullptr");
+            "The StructMethodDeclaration::m_proc_name cannot be nullptr");
 
         SymbolTable *symtab = x.m_parent_symtab;
         require(symtab != nullptr,
-            "ClassProcedure::m_parent_symtab cannot be nullptr");
+            "StructMethodDeclaration::m_parent_symtab cannot be nullptr");
         require(symtab->get_symbol(std::string(x.m_name)) != nullptr,
-            "ClassProcedure '" + std::string(x.m_name) + "' not found in parent_symtab symbol table");
+            "StructMethodDeclaration '" + std::string(x.m_name) + "' not found in parent_symtab symbol table");
         symbol_t *symtab_sym = symtab->get_symbol(std::string(x.m_name));
         const symbol_t *current_sym = &x.base;
         require(symtab_sym == current_sym,
-            "ClassProcedure's parent symbol table does not point to it");
+            "StructMethodDeclaration's parent symbol table does not point to it");
         require(id_symtab_map.find(symtab->counter) != id_symtab_map.end(),
-            "ClassProcedure::m_parent_symtab must be present in the ASR ("
+            "StructMethodDeclaration::m_parent_symtab must be present in the ASR ("
                 + std::string(x.m_name) + ")");
 
         ASR::Function_t* x_m_proc = ASR::down_cast<ASR::Function_t>(x.m_proc);
@@ -424,10 +458,6 @@ public:
     }
 
     void visit_Function(const Function_t &x) {
-        if (ASRUtils::get_FunctionType(&x)->m_abi == abiType::Interactive) {
-            require(x.n_body == 0,
-            "The Function::n_body should be 0 if abi set to Interactive");
-        }
         std::vector<std::string> function_dependencies_copy = function_dependencies;
         function_dependencies.clear();
         function_dependencies.reserve(x.n_dependencies);
@@ -527,7 +557,7 @@ public:
         std::vector<std::string> struct_dependencies;
         for (auto &a : x.m_symtab->get_scope()) {
             this->visit_symbol(*a.second);
-            if( ASR::is_a<ASR::ClassProcedure_t>(*a.second) ||
+            if( ASR::is_a<ASR::StructMethodDeclaration_t>(*a.second) ||
                 ASR::is_a<ASR::GenericProcedure_t>(*a.second) ||
                 ASR::is_a<ASR::Struct_t>(*a.second) ||
                 ASR::is_a<ASR::Union_t>(*a.second) ||
@@ -535,23 +565,21 @@ public:
                 ASR::is_a<ASR::CustomOperator_t>(*a.second) ) {
                 continue ;
             }
+            if ( ASR::is_a<ASR::Variable_t>(*a.second) ) {
+                ASR::Variable_t* var = ASR::down_cast<ASR::Variable_t>(a.second);
+                if ( var->m_type_declaration ) {
+                    struct_dependencies.push_back(
+                        std::string(ASRUtils::symbol_name(var->m_type_declaration)));
+                }
+            }
             // TODO: Uncomment the following line
             // ASR::ttype_t* var_type = ASRUtils::extract_type(ASRUtils::symbol_type(a.second));
             ASR::ttype_t* var_type = ASRUtils::type_get_past_pointer(ASRUtils::symbol_type(a.second));
             char* aggregate_type_name = nullptr;
             ASR::symbol_t* sym = nullptr;
-            if( ASR::is_a<ASR::StructType_t>(*var_type) ) {
-                sym = ASR::down_cast<ASR::StructType_t>(var_type)->m_derived_type;
-                aggregate_type_name = ASRUtils::symbol_name(sym);
-            } else if( ASR::is_a<ASR::EnumType_t>(*var_type) ) {
+            if( ASR::is_a<ASR::EnumType_t>(*var_type) ) {
                 sym = ASR::down_cast<ASR::EnumType_t>(var_type)->m_enum_type;
                 aggregate_type_name = ASRUtils::symbol_name(sym);
-            } else if( ASR::is_a<ASR::UnionType_t>(*var_type) ) {
-                sym = ASR::down_cast<ASR::UnionType_t>(var_type)->m_union_type;
-                aggregate_type_name = ASRUtils::symbol_name(sym);
-            } else if( ASR::is_a<ASR::ClassType_t>(*var_type) ) {
-                sym = ASR::down_cast<ASR::ClassType_t>(var_type)->m_class_type;
-                aggregate_type_name = ASRUtils::symbol_name(sym);
             }
             if( aggregate_type_name && ASRUtils::symbol_parent_symtab(sym) != current_symtab ) {
                 struct_dependencies.push_back(std::string(aggregate_type_name));
@@ -600,7 +628,7 @@ public:
                 "All members of EnumType must have their values to be set. " +
                 std::string(itr_var->m_name) + " doesn't seem to follow this rule in "
                 + std::string(x.m_name) + " EnumType.");
-            require(ASRUtils::check_equal_type(itr_var->m_type, common_type),
+            require(ASRUtils::check_equal_type(itr_var->m_type, common_type, nullptr, nullptr),
                 "All members of EnumType must the same type. " +
                 std::string(itr_var->m_name) + " doesn't seem to follow this rule in " +
                 std::string(x.m_name) + " EnumType.");
@@ -645,6 +673,8 @@ public:
     }
 
     void visit_Variable(const Variable_t &x) {
+        std::string current_name_copy = current_name;
+        current_name = x.m_name;
         variable_dependencies.clear();
         SymbolTable *symtab = x.m_parent_symtab;
         require(symtab != nullptr,
@@ -676,13 +706,55 @@ public:
             !is_module && !is_struct) {
             // For now restrict this check only to variables which are present
             // inside symbols which have a body.
-            require( (x.m_symbolic_value == nullptr && x.m_value == nullptr) ||
-                    (x.m_symbolic_value != nullptr && x.m_value != nullptr) ||
-                    (x.m_symbolic_value != nullptr && ASRUtils::is_value_constant(x.m_symbolic_value)),
-                    "Initialisation of " + std::string(x.m_name) +
-                    " must reduce to a compile time constant.");
-        }
+            ASR::ArrayConstructor_t *array_construct = nullptr;
+            if (x.m_symbolic_value && ASR::is_a<ASR::ArrayConstructor_t>(*x.m_symbolic_value)) {
+                array_construct = ASR::down_cast<ASR::ArrayConstructor_t>(x.m_symbolic_value);
+            }
 
+            if (array_construct && array_construct->n_args > 0) {
+                for (size_t j = 0; j < array_construct->n_args; j++) {
+                    require( (x.m_symbolic_value == nullptr && x.m_value == nullptr) ||
+                            (x.m_symbolic_value != nullptr && x.m_value != nullptr) ||
+                            (x.m_symbolic_value != nullptr && ASRUtils::is_value_constant(array_construct->m_args[j])),
+                            "Initialisation of " + std::string(x.m_name) +
+                            " must reduce to a compile time constant.");
+                }
+            } else {
+                require( (x.m_symbolic_value == nullptr && x.m_value == nullptr) ||
+                        (x.m_symbolic_value != nullptr && x.m_value != nullptr) ||
+                        (x.m_symbolic_value != nullptr && ASRUtils::is_value_constant(x.m_symbolic_value)),
+                        "Initialisation of " + std::string(x.m_name) +
+                        " must reduce to a compile time constant.");
+            }
+        }
+        if(ASRUtils::is_character(*x.m_type)){
+            String_t* str = down_cast<String_t>(ASRUtils::extract_type(x.m_type));
+            require(str->m_len_kind != ASR::ImplicitLength,
+                "Variable symbol of string type can't have a length of kind \"ImplicitLength\"")
+            if(str->m_len_kind == ASR::DeferredLength){
+                /* 
+                    String type Varaible + DeferredLength ==> Must be allocatable or pointer(atleast for Fortran frontend)
+                    String type Expressions + DeferredLength ==> Dont' have to be allocatable or pointer.
+                */ 
+                require(ASRUtils::is_allocatable(x.m_type) || ASRUtils::is_pointer(x.m_type) ,
+                    "Variable of string type with length kind \"DeferredLength\" must be allocatable OR pointer");
+            }
+            if(x.m_abi == abiType::BindC && 
+                x.m_intent != ASR::Local /*Input OR Output*/){
+                if(ASRUtils::is_string_only(x.m_type) && 
+                    str->m_physical_type == CChar){ // Exclude array of strings
+                    require(str->m_len_kind == ASR::ExpressionLength, 
+                        "Cbind character variable that isn't local must have length kind \"ExpressionLength\"");
+                    int64_t len = 0; ASRUtils::extract_value(str->m_len, len);
+                    require(len == 1,
+                        "Cbind character variable that isn't local must have length 1");
+                }
+            }
+            if(str->m_physical_type == ASR::CChar){
+                require(x.m_intent != ASR::Local,
+                    "CChar-string-physical type shouldn't be used with local variables");
+            }
+        }
         if (x.m_symbolic_value)
             visit_expr(*x.m_symbolic_value);
         if (x.m_value)
@@ -712,8 +784,20 @@ public:
                 "Variable " + std::string(x.m_name) + " depends on " +
                 std::string(variable_dependencies[i]) + " but isn't found in its dependency list.");
         }
+        if ( ASR::is_a<ASR::StructType_t>(*ASRUtils::extract_type(x.m_type)) ) {
+            require(x.m_type_declaration != nullptr,
+                "Variable " + std::string(x.m_name) + " of type StructType must have a type declaration.");
+        }
+        current_name = current_name_copy;
     }
 
+    void visit_expr(const expr_t &b){
+        const ASR::expr_t* expr_tmp = current_expr;
+        current_expr = &b;
+        BaseWalkVisitor<VerifyVisitor>::visit_expr(b);
+        current_expr = expr_tmp;
+    }
+    
     void visit_ExternalSymbol(const ExternalSymbol_t &x) {
         if (check_external) {
             require(x.m_external != nullptr,
@@ -792,7 +876,6 @@ public:
     // nodes that have symbol in their fields:
 
     void visit_Var(const Var_t &x) {
-        symbol_visited = true;
         require(x.m_v != nullptr,
             "Var_t::m_v cannot be nullptr");
         std::string x_mv_name = ASRUtils::symbol_name(x.m_v);
@@ -800,13 +883,20 @@ public:
         if (check_external) {
             s = ASRUtils::symbol_get_past_external(x.m_v);
         }
+        if (is_a<ASR::Variable_t>(*s) && is_a<ASR::ExternalSymbol_t>(*x.m_v)) {
+            non_global_symbol_visited = false;
+        } else {
+            non_global_symbol_visited = true;
+        }
         require(is_a<Variable_t>(*s) || is_a<Function_t>(*s)
                 || is_a<ASR::Enum_t>(*s) || is_a<ASR::ExternalSymbol_t>(*s),
             "Var_t::m_v " + x_mv_name + " does not point to a Variable_t, " \
             "Function_t, or Enum_t (possibly behind ExternalSymbol_t)");
         require(symtab_in_scope(current_symtab, x.m_v),
             "Var::m_v `" + x_mv_name + "` cannot point outside of its symbol table");
-        variable_dependencies.push_back(x_mv_name);
+        if ( x_mv_name != current_name ) {
+            variable_dependencies.push_back(x_mv_name);
+        }
     }
 
     void visit_ImplicitDeallocate(const ImplicitDeallocate_t &x) {
@@ -876,6 +966,35 @@ public:
         BaseWalkVisitor<VerifyVisitor>::visit_ArraySize(x);
     }
 
+    void visit_DebugCheckArrayBounds(const ASR::DebugCheckArrayBounds_t& x) {
+        if (check_external) {
+            if (ASR::is_a<ASR::ArraySize_t>(*x.m_target)) {
+                ASR::expr_t* target_v = ASR::down_cast<ASR::ArraySize_t>(x.m_target)->m_v;
+                require(ASR::is_a<ASR::Var_t>(*target_v) ||
+                        ASR::is_a<ASR::ArrayPhysicalCast_t>(*target_v) ||
+                        ASR::is_a<ASR::StructInstanceMember_t>(*target_v) ||
+                        ASR::is_a<ASR::BitCast_t>(*target_v) ||
+                        ASR::is_a<ASR::ArrayConstant_t>(*target_v), "DebugCheckArrayBounds::m_target::m_v must be Var, ArrayPhysicalCast, StructInstanceMember, BitCast, or ArrayConstant");
+                require(ASRUtils::is_array(ASRUtils::type_get_past_allocatable_pointer(ASRUtils::expr_type(target_v))), "DebugCheckArrayBounds::m_target::m_v must have an Array type");
+            } else {
+                require(ASRUtils::is_integer(*ASRUtils::expr_type(x.m_target)), "DebugCheckArrayBounds::m_target must have an Integer type")
+            }
+
+            if (ASR::is_a<ASR::ArraySize_t>(*x.m_value)) {
+                ASR::expr_t* value_v = ASR::down_cast<ASR::ArraySize_t>(x.m_value)->m_v;
+                require(ASR::is_a<ASR::Var_t>(*value_v) ||
+                        ASR::is_a<ASR::ArrayPhysicalCast_t>(*value_v) ||
+                        ASR::is_a<ASR::StructInstanceMember_t>(*value_v) ||
+                        ASR::is_a<ASR::BitCast_t>(*value_v) ||
+                        ASR::is_a<ASR::ArrayConstant_t>(*value_v), "DebugCheckArrayBounds::m_value::m_v must be Var, ArrayPhysicalCast, StructInstanceMember, BitCast, or ArrayConstant");
+                require(ASRUtils::is_array(ASRUtils::type_get_past_allocatable_pointer(ASRUtils::expr_type(value_v))), "DebugCheckArrayBounds::m_value::m_v must have an Array type");
+            } else {
+                require(ASRUtils::is_integer(*ASRUtils::expr_type(x.m_value)), "DebugCheckArrayBounds::m_value must have an Integer type")
+            }
+        }
+        BaseWalkVisitor<VerifyVisitor>::visit_DebugCheckArrayBounds(x);
+    }
+
     void visit_ArraySection(const ArraySection_t &x) {
         require(
             ASR::is_a<ASR::Array_t>(*x.m_type),
@@ -908,11 +1027,14 @@ public:
             }
         }
 
+        bool _inside_call_copy = _inside_call;
+        _inside_call = true;
         for (size_t i=0; i<x.n_args; i++) {
             if( x.m_args[i].m_value ) {
                 visit_expr(*(x.m_args[i].m_value));
             }
         }
+        _inside_call = _inside_call_copy;
     }
 
     void visit_ArrayPhysicalCast(const ASR::ArrayPhysicalCast_t& x) {
@@ -927,6 +1049,10 @@ public:
             require(x.m_old == ASRUtils::extract_physical_type(ASRUtils::expr_type(x.m_arg)),
                 "Old physical type conflicts with the physical type of argument " + std::to_string(x.m_old)
                 + " " + std::to_string(ASRUtils::extract_physical_type(ASRUtils::expr_type(x.m_arg))));
+            bool _inside_array_physical_cast_type_copy = _inside_array_physical_cast_type;
+            _inside_array_physical_cast_type = true;
+            visit_ttype(*x.m_type);
+            _inside_array_physical_cast_type = _inside_array_physical_cast_type_copy;
         }
     }
 
@@ -943,8 +1069,8 @@ public:
                     "SubroutineCall::m_name '" + std::string(symbol_name(x.m_name)) + "' is a Variable, but the type is not FunctionType");
             } else {
                 require(ASR::is_a<ASR::Function_t>(*s) ||
-                        ASR::is_a<ASR::ClassProcedure_t>(*s),
-                    "SubroutineCall::m_name '" + std::string(symbol_name(x.m_name)) + "' must be a Function or ClassProcedure.");
+                        ASR::is_a<ASR::StructMethodDeclaration_t>(*s),
+                    "SubroutineCall::m_name '" + std::string(symbol_name(x.m_name)) + "' must be a Function or StructMethodDeclaration.");
             }
         }
 
@@ -983,35 +1109,6 @@ public:
                 "' cannot point outside of its symbol table");
     }
 
-    SymbolTable *get_dt_symtab(ASR::symbol_t *dt) {
-        LCOMPILERS_ASSERT(dt)
-        SymbolTable *symtab = ASRUtils::symbol_symtab(ASRUtils::symbol_get_past_external(dt));
-        require_with_loc(symtab,
-            "m_dt::m_v::m_type::class/derived_type must point to a symbol with a symbol table",
-            dt->base.loc);
-        return symtab;
-    }
-
-    SymbolTable *get_dt_symtab(ASR::expr_t *dt) {
-        ASR::ttype_t *t2 = ASRUtils::type_get_past_pointer(ASRUtils::expr_type(dt));
-        ASR::symbol_t *type_sym=nullptr;
-        switch (t2->type) {
-            case (ASR::ttypeType::StructType): {
-                type_sym = ASR::down_cast<ASR::StructType_t>(t2)->m_derived_type;
-                break;
-            }
-            case (ASR::ttypeType::ClassType): {
-                type_sym = ASR::down_cast<ASR::ClassType_t>(t2)->m_class_type;
-                break;
-            }
-            default :
-                require_with_loc(false,
-                    "m_dt::m_v::m_type must point to a type with a symbol table (StructType or ClassType)",
-                    dt->base.loc);
-        }
-        return get_dt_symtab(type_sym);
-    }
-
     ASR::symbol_t *get_parent_type_dt(ASR::symbol_t *dt) {
         ASR::symbol_t *parent = nullptr;
         switch (dt->type) {
@@ -1029,44 +1126,18 @@ public:
         return parent;
     }
 
-    ASR::symbol_t *get_parent_type_dt(ASR::expr_t *dt) {
-        ASR::ttype_t *t2 = ASRUtils::type_get_past_pointer(ASRUtils::expr_type(dt));
-        ASR::symbol_t *type_sym=nullptr;
-        ASR::symbol_t *parent = nullptr;
-        switch (t2->type) {
-            case (ASR::ttypeType::StructType): {
-                type_sym = ASR::down_cast<ASR::StructType_t>(t2)->m_derived_type;
-                type_sym = ASRUtils::symbol_get_past_external(type_sym);
-                ASR::Struct_t* der_type = ASR::down_cast<ASR::Struct_t>(type_sym);
-                parent = der_type->m_parent;
-                break;
-            }
-            case (ASR::ttypeType::ClassType): {
-                type_sym = ASR::down_cast<ASR::ClassType_t>(t2)->m_class_type;
-                type_sym = ASRUtils::symbol_get_past_external(type_sym);
-                if( type_sym->type == ASR::symbolType::Struct ) {
-                    ASR::Struct_t* der_type = ASR::down_cast<ASR::Struct_t>(type_sym);
-                    parent = der_type->m_parent;
-                }
-                break;
-            }
-            default :
-                require_with_loc(false,
-                    "m_dt::m_v::m_type must point to a StructType type",
-                    dt->base.loc);
-        }
-        return parent;
-    }
-
     void visit_PointerNullConstant(const PointerNullConstant_t& x) {
         require(x.m_type != nullptr, "null() must have a type");
+        if ( x.m_var_expr != nullptr ) {
+            visit_expr(*x.m_var_expr);
+        }
     }
 
     void visit_FunctionType(const FunctionType_t& x) {
 
-        #define verify_nonscoped_ttype(ttype) symbol_visited = false; \
+        #define verify_nonscoped_ttype(ttype) non_global_symbol_visited = false; \
             visit_ttype(*ttype); \
-            require(symbol_visited == false, \
+            require(non_global_symbol_visited == false, \
                     "ASR::ttype_t in ASR::FunctionType" \
                     " cannot be tied to a scope."); \
 
@@ -1087,7 +1158,10 @@ public:
             ::get_verify_function(x.m_intrinsic_id);
         LCOMPILERS_ASSERT(verify_ != nullptr);
         verify_(x, diagnostics);
+        bool _inside_call_copy = _inside_call;
+        _inside_call = true;
         BaseWalkVisitor<VerifyVisitor>::visit_IntrinsicElementalFunction(x);
+        _inside_call = _inside_call_copy;
     }
 
     void visit_IntrinsicArrayFunction(const ASR::IntrinsicArrayFunction_t& x) {
@@ -1099,12 +1173,16 @@ public:
             ::get_verify_function(x.m_arr_intrinsic_id);
         LCOMPILERS_ASSERT(verify_ != nullptr);
         verify_(x, diagnostics);
+        bool _inside_call_copy = _inside_call;
+        _inside_call = true;
         BaseWalkVisitor<VerifyVisitor>::visit_IntrinsicArrayFunction(x);
+        _inside_call = _inside_call_copy;
     }
 
     void visit_FunctionCall(const FunctionCall_t &x) {
         require(x.m_name,
             "FunctionCall::m_name must be present");
+        variable_dependencies.push_back(std::string(ASRUtils::symbol_name(x.m_name)));
         ASR::symbol_t* asr_owner_sym = nullptr;
         if(current_symtab->asr_owner &&  ASR::is_a<ASR::symbol_t>(*current_symtab->asr_owner) ) {
             asr_owner_sym = ASR::down_cast<ASR::symbol_t>(current_symtab->asr_owner);
@@ -1147,7 +1225,7 @@ public:
             require(ASR::is_a<ASR::Function_t>(*fn) ||
                     (ASR::is_a<ASR::Variable_t>(*fn) &&
                     ASR::is_a<ASR::FunctionType_t>(*ASRUtils::symbol_type(fn))) ||
-                    ASR::is_a<ASR::ClassProcedure_t>(*fn),
+                    ASR::is_a<ASR::StructMethodDeclaration_t>(*fn),
                 "FunctionCall::m_name must be a Function or Variable with FunctionType");
         }
 
@@ -1161,16 +1239,10 @@ public:
         visit_ttype(*x.m_type);
     }
 
-    void visit_StructType(const StructType_t &x) {
-        std::string symbol_owner = "global scope";
-        if( ASRUtils::get_asr_owner(x.m_derived_type) ) {
-            symbol_owner = ASRUtils::symbol_name(ASRUtils::get_asr_owner(x.m_derived_type));
+    void visit_StructType(const StructType_t& x) {
+        for (size_t i = 0; i < x.n_data_member_types; i++) {
+            visit_ttype(*x.m_data_member_types[i]);
         }
-        require(symtab_in_scope(current_symtab, x.m_derived_type),
-            "StructType::m_derived_type '" +
-            std::string(ASRUtils::symbol_name(x.m_derived_type)) +
-            "' cannot point outside of its symbol table, owner: " +
-            symbol_owner);
     }
 
     void visit_ArrayConstructor(const ArrayConstructor_t& x) {
@@ -1186,13 +1258,20 @@ public:
         int64_t n_data = ASRUtils::get_fixed_size_of_array(x.m_type) * ASRUtils::extract_kind_from_ttype_t(x.m_type);
         if (ASRUtils::is_character(*x.m_type)) {
             ASR::ttype_t* t = ASRUtils::type_get_past_array(x.m_type);
-            n_data = ASRUtils::get_fixed_size_of_array(x.m_type) * ASR::down_cast<ASR::String_t>(t)->m_len;
+            int64_t len;
+            require(ASRUtils::extract_value(ASR::down_cast<ASR::String_t>(t)->m_len, len), "Constant array of strings should have constant string length");
+            n_data = ASRUtils::get_fixed_size_of_array(x.m_type) * len;
         }
         require(n_data == x.m_n_data, "ArrayConstant::m_n_data must match the byte size of the array");
         visit_ttype(*x.m_type);
     }
 
     void visit_dimension(const dimension_t &x) {
+        if (_inside_array_physical_cast_type && !_inside_call) {
+            require_with_loc(x.m_length != nullptr && x.m_start != nullptr,
+                    "Dimensions in ArrayPhysicalCast must be present if not inside a call",
+                    x.loc);
+        }
         if (x.m_start) {
             if(check_external){
                 require_with_loc(ASRUtils::is_integer(
@@ -1218,6 +1297,10 @@ public:
         visit_ttype(*x.m_type);
         require(x.n_dims != 0, "Array type cannot have 0 dimensions.")
         require(!ASR::is_a<ASR::Array_t>(*x.m_type), "Array type cannot be nested.")
+        if(ASRUtils::is_character(*x.m_type)){
+            require(x.m_physical_type != ASR::FixedSizeArray,
+                "Array of strings' physical type shouldn't be \"FixedSizeArray\"")
+        }
         _processing_dims = true;
         for (size_t i = 0; i < x.n_dims; i++) {
             visit_dimension(x.m_dims[i]);
@@ -1240,7 +1323,8 @@ public:
     }
 
     void visit_Allocatable(const Allocatable_t &x) {
-        require(!ASR::is_a<ASR::Pointer_t>(*x.m_type),
+        require(!ASR::is_a<ASR::Pointer_t>(*x.m_type) &&
+                !ASR::is_a<ASR::Allocatable_t>(*x.m_type),
             "Allocatable type conflicts with Pointer type");
         ASR::dimension_t* m_dims = nullptr;
         size_t n_dims = ASRUtils::extract_dimensions_from_ttype(x.m_type, m_dims);
@@ -1251,6 +1335,70 @@ public:
         visit_ttype(*x.m_type);
     }
 
+    void visit_String(const String_t &x){
+/*General Check on the length*/ 
+        if(x.m_len){
+            require(ASR::is_a<ASR::Integer_t>(*ASRUtils::expr_type(x.m_len)),
+                "String length must be of type INTEGER,"
+                "found " + 
+                ASRUtils::type_to_str_fortran_expr(ASRUtils::expr_type(x.m_len), x.m_len));
+        }
+// Check Positive Length
+        if(x.m_len && ASRUtils::is_value_constant(x.m_len)){
+            int64_t len{};
+            ASRUtils::is_value_constant(x.m_len, len);
+            require(len >= 0,
+                "String length must be length >= 0\nCurrent length is -> " + std::to_string(len));
+        }
+/*Check Valid String type state based on the physical type*/
+        if (x.m_physical_type == DescriptorString ||
+            x.m_physical_type == CChar){
+            std::string type_as_str = (x.m_physical_type == DescriptorString) ? "\"DescriptorString\"" : "\"CChar\"";
+            if(x.m_len){
+                require(x.m_len_kind == ExpressionLength,
+                    "String of physical type " +
+                    type_as_str +
+                    " + existing length => must have length kind of \"ExpressionLength\".")
+            } else {
+                require(x.m_len_kind == AssumedLength ||
+                        x.m_len_kind == DeferredLength ||
+                        x.m_len_kind == ImplicitLength,
+                    "String of physical type " +
+                    type_as_str +
+                    " + non-existing length => must have length kind of"
+                    " \"AssumedLength\" OR \"DeferredLength\" OR \"ImplicitLength\".")
+            }
+        } else {
+            throw LCompilersException("PhysicalType not checked (Probably a new physical type).");
+        }
+/*Check if implicitLength is used correctly*/
+        if(x.m_len_kind == ASR::ImplicitLength){
+            require(current_expr && ASR::is_a<ASR::StringPhysicalCast_t>(*current_expr),
+                "Implicit length kind must appear in StringPhysicalCast expression.");
+        }
+        BaseWalkVisitor<VerifyVisitor>::visit_String(x);
+    }
+    void visit_StringPhysicalCast(const StringPhysicalCast_t &x){
+        require(x.m_type, "x.m_type cannot be nullptr");
+        require(ASR::is_a<ASR::String_t>(*x.m_type), "StringPhysicalCast should be of string type");
+        ASR::String_t* str = ASR::down_cast<ASR::String_t>(x.m_type);
+        require(!str->m_len,
+            "StringPhysicalCast return type shouldn't have length "
+            "(Length should be implicit).")
+        require(str->m_len_kind == ImplicitLength,
+            "StringPhysicalCast expression should have length kind of \"ImplicitLength\".")
+        BaseWalkVisitor<VerifyVisitor>::visit_StringPhysicalCast(x);
+    }
+    void visit_StringSection(const StringSection_t &x){
+        require(x.m_start, "StringSection start member must be provided")
+        require(x.m_end, "StringSection end member must be provided")
+        require(x.m_step, "StringSection step member must be provided")
+        require(ASR::is_a<ASR::String_t>(*x.m_type), "StringSection return type must be a string")
+        require(ASRUtils::get_string_type(x.m_type)->m_len, "StringSection's string-return node must have length expression (NOT nullptr)")
+        BaseWalkVisitor<VerifyVisitor>::visit_StringSection(x);
+    }
+
+
     void visit_Allocate(const Allocate_t &x) {
         if(check_external){
             for( size_t i = 0; i < x.n_args; i++ ) {
@@ -1258,8 +1406,32 @@ public:
                         ASR::is_a<ASR::Pointer_t>(*ASRUtils::expr_type(x.m_args[i].m_a)),
                     "Allocate should only be called with  Allocatable or Pointer type inputs, found " +
                     std::string(ASRUtils::get_type_code(ASRUtils::expr_type(x.m_args[i].m_a))));
+                ASR::ttype_t* alloc_arg_type = x.m_args[i].m_type;
+                if ( alloc_arg_type && ASRUtils::is_struct(*alloc_arg_type) && x.m_args[i].m_sym_subclass != nullptr) {
+                    require(ASR::is_a<ASR::Struct_t>(*ASRUtils::symbol_get_past_external(x.m_args[i].m_sym_subclass)),
+                        "Allocate::m_sym_subclass must point to a Struct_t when the m_a member is of a type StructType");
+                }
+                // Check Allocating a string OR an array of string with deferred length
+                // Not providing length in Allocate statement with non-deferredLength is permissible
+                if(!x.m_source &&
+                    ASRUtils::is_character(*ASRUtils::expr_type(x.m_args[i].m_a)) && 
+                    ASRUtils::get_string_type(ASRUtils::expr_type(x.m_args[i].m_a))->m_len_kind == ASR::DeferredLength){
+                    require(x.m_args[i].m_len_expr,
+                        "Allocating a variable that's a string of deferred length requires providing a length to allocate with");
+                }
+            }
+
+            if( x.m_source == nullptr ) {
+                for( size_t i = 0; i < x.n_args; i++ ) {
+                    if( ASRUtils::is_array(ASRUtils::expr_type(x.m_args[i].m_a)) ) {
+                        require(x.m_args[i].n_dims > 0,
+                            "Allocate for arrays should have dimensions specified, "
+                            "found only array variable with no dimensions");
+                    }
+                }
             }
         }
+
         BaseWalkVisitor<VerifyVisitor>::visit_Allocate(x);
     }
 
