@@ -92,6 +92,15 @@ public:
             } case (ASR::binopType::Pow) : {
                 last_expr_precedence = Precedence::Pow;
                 return "**";
+            } case (ASR::binopType::BitOr) : {
+                last_expr_precedence = Precedence::Or;
+                return " .or. ";
+            } case (ASR::binopType::BitAnd) : {
+                last_expr_precedence = Precedence::And;
+                return " .and. ";
+            } case (ASR::binopType::BitXor) : {
+                last_expr_precedence = Precedence::Eqv;
+                return " .eqv. ";
             } default : {
                 throw LCompilersException("Binop type not implemented");
             }
@@ -180,16 +189,23 @@ public:
             } case ASR::ttypeType::String: {
                 ASR::String_t *c = down_cast<ASR::String_t>(t);
                 r = "character(len=";
-                if(c->m_len > 0) {
-                    r += std::to_string(c->m_len);
+                int str_len;
+                if(ASRUtils::extract_value(c->m_len, str_len)) {
+                    r += std::to_string(str_len);
                 } else {
-                    if (c->m_len == -1) {
-                        r += "*";
-                    } else if (c->m_len == -2) {
-                        r += ":";
-                    } else if (c->m_len == -3) {
-                        visit_expr(*c->m_len_expr);
-                        r += src;
+                    switch(c->m_len_kind){
+                        case ASR::string_length_kindType::ExpressionLength:
+                            visit_expr(*c->m_len);
+                            r += src;
+                            break;
+                        case ASR::string_length_kindType::AssumedLength:
+                            r += "*";
+                            break;
+                        case ASR::string_length_kindType::DeferredLength:
+                            r += ":";
+                            break;
+                        case ASR::ImplicitLength:
+                            throw LCompilersException("Shouldn't be visited (Can't deduce its length, It's not a variable).");
                     }
                 }
                 r += ", kind=";
@@ -217,7 +233,11 @@ public:
                     }
 
                     if (len.length() == 0) {
-                        bounds += ":";
+                        if ( start.length() != 0 ) {
+                            bounds += start + ":";
+                        } else {
+                            bounds += ":";
+                        }
                     } else {
                         if (start.length() == 0 || start == "1") {
                             bounds += len;
@@ -226,17 +246,16 @@ public:
                         }
                     }
                 }
-                r = get_type(arr_type->m_type) + ", dimension(" + bounds + ")";
+                r = get_type(arr_type->m_type, type_decl) + ", dimension(" + bounds + ")";
                 break;
             } case ASR::ttypeType::Allocatable: {
-                r = get_type(down_cast<ASR::Allocatable_t>(t)->m_type) + ", allocatable";
+                r = get_type(down_cast<ASR::Allocatable_t>(t)->m_type, type_decl) + ", allocatable";
                 break;
             } case ASR::ttypeType::Pointer: {
-                r = get_type(down_cast<ASR::Pointer_t>(t)->m_type) + ", pointer";
+                r = get_type(down_cast<ASR::Pointer_t>(t)->m_type, type_decl) + ", pointer";
                 break;
             } case ASR::ttypeType::StructType: {
-                ASR::StructType_t* struct_type = down_cast<ASR::StructType_t>(t);
-                std::string struct_name = ASRUtils::symbol_name(struct_type->m_derived_type);
+                std::string struct_name = ASRUtils::symbol_name(type_decl);
                 r = "type(";
                 r += struct_name;
                 r += ")";
@@ -257,7 +276,7 @@ public:
             }
             default:
                 throw LCompilersException("The type `"
-                    + ASRUtils::type_to_str_python(t) + "` is not handled yet");
+                    + ASRUtils::type_to_str_python_symbol(t, type_decl) + "` is not handled yet");
         }
         return r;
     }
@@ -278,12 +297,15 @@ public:
     /********************************** Unit **********************************/
     void visit_TranslationUnit(const ASR::TranslationUnit_t &x) {
         std::string r = "";
-        for (auto &item : x.m_symtab->get_scope()) {
-            if (is_a<ASR::Module_t>(*item.second)) {
-                visit_symbol(*item.second);
-                r += src;
-                r += "\n";
-            }
+        std::vector<std::string> build_order
+            = ASRUtils::determine_module_dependencies(x);
+        for (auto &item : build_order) {
+            LCOMPILERS_ASSERT(x.m_symtab->get_symbol(item)
+                != nullptr);
+            ASR::symbol_t *mod = x.m_symtab->get_symbol(item);
+            visit_symbol(*mod);
+            r += src;
+            r += "\n";
         }
 
         tu_functions = "";
@@ -490,9 +512,12 @@ public:
         }
         bool is_return_var_declared = false;
         if (x.m_return_var) {
-            if (!ASRUtils::is_array(ASRUtils::expr_type(x.m_return_var))) {
+            if (!ASRUtils::is_array(ASRUtils::expr_type(x.m_return_var)) &&
+                !ASRUtils::is_allocatable(ASRUtils::expr_type(x.m_return_var)) &&
+                !ASRUtils::is_pointer(ASRUtils::expr_type(x.m_return_var))) {
                 is_return_var_declared = true;
-                r += get_type(ASRUtils::expr_type(x.m_return_var));
+                ASR::Variable_t *return_var = ASRUtils::EXPR2VAR(x.m_return_var);
+                r += get_type(return_var->m_type, return_var->m_type_declaration);
                 r += " ";
             }
             r += "function";
@@ -646,7 +671,16 @@ public:
 
     void visit_Struct(const ASR::Struct_t &x) {
         std::string r = indent;
-        r += "type :: ";
+        r += "type";
+        if (x.m_parent) {
+            r += ", extends(";
+            r += std::string(ASR::down_cast<ASR::Struct_t>(x.m_parent)->m_name);
+            r += ")";
+        }
+        if (x.m_is_abstract) {
+            r += ", abstract";
+        }
+        r += " :: ";
         r.append(x.m_name);
         handle_line_truncation(r, 2);
         r += "\n";
@@ -659,6 +693,17 @@ public:
                 r += src;
             }
         }
+
+        std::vector<std::string> class_procedure_order = ASRUtils::determine_class_procedure_declaration_order(x.m_symtab);
+        if (class_procedure_order.size() > 0) r += "contains\n";
+        for (auto &item : class_procedure_order) {
+            ASR::symbol_t* class_procedure_sym = x.m_symtab->get_symbol(item);
+            if (is_a<ASR::StructMethodDeclaration_t>(*class_procedure_sym)) {
+                visit_symbol(*class_procedure_sym);
+                r += src;
+            }
+        }
+
         dec_indent();
         r += "end type ";
         r.append(x.m_name);
@@ -736,7 +781,11 @@ public:
             visit_expr(*x.m_symbolic_value);
             r += src;
         } else if (x.m_value) {
-            r += " = ";
+            if (ASR::is_a<ASR::PointerNullConstant_t>(*x.m_value)) {
+                r += " => ";
+            } else {
+                r += " = ";
+            }
             visit_expr(*x.m_value);
             r += src;
         } else if (x.m_symbolic_value) {
@@ -749,13 +798,73 @@ public:
         src = r;
     }
 
-    // void visit_Class(const ASR::Class_t &x) {}
-
-    // void visit_ClassProcedure(const ASR::ClassProcedure_t &x) {}
+    void visit_StructMethodDeclaration(const ASR::StructMethodDeclaration_t &x) {
+        std::string r = indent;
+        r += "procedure";
+        if (x.m_is_deferred) {
+            if (strcmp(x.m_name, x.m_proc_name)) {
+                r += "(";
+                r += std::string(x.m_proc_name);
+                r += ")";
+            }
+            r += ", deferred";
+            r += " :: ";
+            r += std::string(x.m_name);
+        } else {
+            r += " :: ";
+            r += std::string(x.m_name);
+            if (strcmp(x.m_name, x.m_proc_name)) {
+                r += " => ";
+                r += std::string(x.m_proc_name);
+            }
+        }
+        r += "\n";
+        src = r;
+    }
 
     // void visit_AssociateBlock(const ASR::AssociateBlock_t &x) {}
 
-    // void visit_Block(const ASR::Block_t &x) {}
+    void visit_Block(const ASR::Block_t &x) {
+        // Generate Fortran for a Block symbol in place
+        std::string r = indent;
+        if (x.m_name) {
+            r += std::string(x.m_name) + " : ";
+        }
+        r += "block\n";
+
+        inc_indent();
+        // Emit any use statements from this scope
+        for (auto &item : x.m_symtab->get_scope()) {
+            if (is_a<ASR::ExternalSymbol_t>(*item.second)) {
+                visit_symbol(*item.second);
+                r += src;
+            }
+        }
+        // Emit local variable declarations
+        std::vector<std::string> var_order = ASRUtils::determine_variable_declaration_order(x.m_symtab);
+        for (auto &item : var_order) {
+            ASR::symbol_t* var_sym = x.m_symtab->get_symbol(item);
+            if (is_a<ASR::Variable_t>(*var_sym)) {
+                visit_symbol(*var_sym);
+                r += src;
+            }
+        }
+
+        // Emit the block body
+        for (size_t i = 0; i < x.n_body; i++) {
+            visit_stmt(*x.m_body[i]);
+            r += src;
+        }
+        dec_indent();
+
+        r += indent;
+        r += "end block";
+        if (x.m_name) {
+            r += " " + std::string(x.m_name);
+        }
+        r += "\n";
+        src = r;
+    }
 
     // void visit_Requirement(const ASR::Requirement_t &x) {}
 
@@ -766,6 +875,16 @@ public:
         std::string r = indent;
         r += "allocate(";
         for (size_t i = 0; i < x.n_args; i ++) {
+            if (x.m_args[i].m_type) {
+                r += get_type(x.m_args[i].m_type, x.m_args[i].m_sym_subclass);
+                r += " :: ";
+            }
+            if (x.m_args[i].m_len_expr) {
+                r += "character(len=";
+                visit_expr(*x.m_args[i].m_len_expr);
+                r += src;
+                r += ") :: ";
+            }
             visit_expr(*x.m_args[i].m_a);
             r += src;
             if (x.m_args[i].n_dims > 0) {
@@ -779,6 +898,11 @@ public:
             }
             if (i < x.n_args-1) r += ", ";
         }
+        if (x.m_stat) {
+            r += ", stat=";
+            visit_expr(*x.m_stat);
+            r += src;
+        }
         r += ")";
         handle_line_truncation(r, 2);
         r += "\n";
@@ -813,6 +937,19 @@ public:
         src = r;
     }
 
+    void visit_DebugCheckArrayBounds(const ASR::DebugCheckArrayBounds_t &x) {
+        std::string r = indent;
+        r += "! CheckBounds LHS = ";
+        visit_expr(*x.m_target);
+        r += src;
+        r += ", RHS = ";
+        visit_expr(*x.m_value);
+        r += src;
+        handle_line_truncation(r, 2);
+        r += "\n";
+        src = r;
+    }
+
     void visit_Associate(const ASR::Associate_t &x) {
         visit_expr(*x.m_target);
         std::string t = std::move(src);
@@ -1214,14 +1351,18 @@ public:
     void visit_SubroutineCall(const ASR::SubroutineCall_t &x) {
         std::string r = indent;
         r += "call ";
-        r += ASRUtils::symbol_name(x.m_name);
+        if (x.m_dt) {
+            visit_expr(*x.m_dt);
+            r += src + "%";
+        }
+        r += ASRUtils::symbol_name(ASRUtils::symbol_get_past_external(x.m_name));
         r += "(";
         for (size_t i = 0; i < x.n_args; i ++) {
             visit_expr(*x.m_args[i].m_value);
             r += src;
             if (i < x.n_args-1) r += ", ";
         }
-        
+
         r += ")";
         handle_line_truncation(r, 2);
         r += "\n";
@@ -1297,13 +1438,122 @@ public:
 
     // void visit_Flush(const ASR::Flush_t &x) {}
 
-    // void visit_AssociateBlockCall(const ASR::AssociateBlockCall_t &x) {}
+    void visit_AssociateBlockCall(const ASR::AssociateBlockCall_t &x) {
+        LCOMPILERS_ASSERT(ASR::is_a<ASR::AssociateBlock_t>(*x.m_m));
+        ASR::AssociateBlock_t* ab = ASR::down_cast<ASR::AssociateBlock_t>(x.m_m);
+
+        // Build the association header from leading Associate statements
+        std::vector<std::pair<std::string, std::string>> assoc_pairs;
+        size_t body_start = 0;
+        for (; body_start < ab->n_body; body_start++) {
+            ASR::stmt_t* s = ab->m_body[body_start];
+            if (ASR::is_a<ASR::Associate_t>(*s)) {
+                ASR::Associate_t* as = ASR::down_cast<ASR::Associate_t>(s);
+                visit_expr(*as->m_target);
+                std::string t = src;
+                visit_expr(*as->m_value);
+                std::string v = src;
+                assoc_pairs.emplace_back(std::move(t), std::move(v));
+            } else {
+                break;
+            }
+        }
+
+        std::string r = indent;
+        if (ab->m_name) {
+            r += std::string(ab->m_name) + " : ";
+        }
+        r += "associate (";
+        for (size_t i = 0; i < assoc_pairs.size(); i++) {
+            r += assoc_pairs[i].first + " => " + assoc_pairs[i].second;
+            if (i + 1 < assoc_pairs.size()) r += ", ";
+        }
+        r += ")\n";
+
+        inc_indent();
+
+        // Emit any use statements from this scope
+        for (auto &item : ab->m_symtab->get_scope()) {
+            if (is_a<ASR::ExternalSymbol_t>(*item.second)) {
+                visit_symbol(*item.second);
+                r += src;
+            }
+        }
+        // Emit variable declarations that are not associate names
+        std::set<std::string> assoc_names;
+        for (auto &p: assoc_pairs) assoc_names.insert(p.first);
+        std::vector<std::string> var_order = ASRUtils::determine_variable_declaration_order(ab->m_symtab);
+        for (auto &item : var_order) {
+            if (assoc_names.find(item) != assoc_names.end()) continue;
+            ASR::symbol_t* var_sym = ab->m_symtab->get_symbol(item);
+            if (is_a<ASR::Variable_t>(*var_sym)) {
+                visit_symbol(*var_sym);
+                r += src;
+            }
+        }
+
+        // Emit the rest of the body (after the Associate statements)
+        for (size_t i = body_start; i < ab->n_body; i++) {
+            visit_stmt(*ab->m_body[i]);
+            r += src;
+        }
+        dec_indent();
+        r += indent + "end associate";
+        if (ab->m_name) {
+            r += " " + std::string(ab->m_name);
+        }
+        r += "\n";
+        src = r;
+    }
 
     // void visit_SelectType(const ASR::SelectType_t &x) {}
 
     // void visit_CPtrToPointer(const ASR::CPtrToPointer_t &x) {}
 
-    // void visit_BlockCall(const ASR::BlockCall_t &x) {}
+    void visit_BlockCall(const ASR::BlockCall_t &x) {
+        LCOMPILERS_ASSERT(ASR::is_a<ASR::Block_t>(*x.m_m));
+        ASR::Block_t* block = ASR::down_cast<ASR::Block_t>(x.m_m);
+        std::string r = indent;
+        if (block->m_name) {
+            r += std::string(block->m_name) + " : ";
+        }
+        r += "block\n";
+
+        inc_indent();
+        // Declarations from this block's scope
+        for (auto &item : block->m_symtab->get_scope()) {
+            if (is_a<ASR::ExternalSymbol_t>(*item.second)) {
+                visit_symbol(*item.second);
+                r += src;
+            }
+        }
+        std::vector<std::string> var_order = ASRUtils::determine_variable_declaration_order(block->m_symtab);
+        for (auto &item : var_order) {
+            ASR::symbol_t* var_sym = block->m_symtab->get_symbol(item);
+            if (is_a<ASR::Variable_t>(*var_sym)) {
+                visit_symbol(*var_sym);
+                r += src;
+            }
+        }
+        // If this block is a goto target (used by select-case lowering), emit the label target
+        if (x.m_label != -1) {
+            r += std::to_string(x.m_label) + " continue\n";
+        }
+        // Emit body
+        for (size_t i = 0; i < block->n_body; i++) {
+            visit_stmt(*block->m_body[i]);
+            r += src;
+        }
+        dec_indent();
+
+        r += indent;
+        r += "end block";
+        if (block->m_name) {
+            r += " " + std::string(block->m_name);
+        }
+        r += "\n";
+        src = r;
+    }
 
     // void visit_Expr(const ASR::Expr_t &x) {}
 
@@ -1336,7 +1586,11 @@ public:
         if (x.m_original_name) {
             r += ASRUtils::symbol_name(x.m_original_name);
         } else {
-            r += ASRUtils::symbol_name(x.m_name);
+            if (x.m_dt) {
+                visit_expr(*x.m_dt);
+                r += src + "%";
+            }
+            r += ASRUtils::symbol_name(ASRUtils::symbol_get_past_external(x.m_name));
         }
         r += "(";
         for (size_t i = 0; i < x.n_args; i ++) {
@@ -1362,6 +1616,8 @@ public:
             SET_INTRINSIC_SUBROUTINE_NAME(Srand, "srand");
             SET_INTRINSIC_SUBROUTINE_NAME(SystemClock, "system_clock");
             SET_INTRINSIC_SUBROUTINE_NAME(DateAndTime, "date_and_time");
+            SET_INTRINSIC_SUBROUTINE_NAME(MoveAlloc, "move_alloc");
+            SET_INTRINSIC_SUBROUTINE_NAME(Mvbits, "mvbits")
             default : {
                 throw LCompilersException("IntrinsicImpureSubroutine: `"
                     + ASRUtils::get_intrinsic_name(x.m_sub_intrinsic_id)
@@ -1379,6 +1635,13 @@ public:
     }
 
     void visit_IntrinsicElementalFunction_helper(std::string &out, std::string func_name, const ASR::IntrinsicElementalFunction_t &x) {
+        if ( x.m_intrinsic_id == static_cast<int64_t>(ASRUtils::IntrinsicElementalFunctions::CompilerVersion) ) {
+            src = "";
+            visit_expr(*x.m_value); // we will always have a value
+            out += src;
+            src = out;
+            return;
+        }
         src = "";
         out += func_name;
         if (x.n_args > 0) visit_expr(*x.m_args[0]);
@@ -1411,6 +1674,7 @@ public:
         else if(intrinsic_func_name == "CompilerVersion") intrinsic_func_name = "compiler_version";
         else if(intrinsic_func_name == "CommandArgumentCount") intrinsic_func_name = "command_argument_count";
         else if(intrinsic_func_name == "ErfcScaled") intrinsic_func_name = "erfc_scaled";
+        else if(intrinsic_func_name == "StringConcat") {{visit_expr(*x.m_args[0]);out+=src;} out+="//"; {visit_expr(*x.m_args[1]);out+=src;} src=std::move(out);return;}
         visit_IntrinsicElementalFunction_helper(out, intrinsic_func_name, x);
     }
 
@@ -1455,12 +1719,30 @@ public:
             } else if(ASR::is_a<ASR::ArrayConstructor_t>(*arr_physical->m_arg)){
                 ASR::ArrayConstructor_t *arr_const = ASR::down_cast<ASR::ArrayConstructor_t>(arr_physical->m_arg);
                 x.m_args[0] = arr_const->m_args[0];
-            }           
+            }
         }
         visit_IntrinsicArrayFunction_helper(out, intrinsic_func_name, x);
     }
 
-    // void visit_IntrinsicImpureFunction(const ASR::IntrinsicImpureFunction_t &x) {}
+    void visit_IntrinsicImpureFunction_helper(std::string &out, std::string func_name, const ASR::IntrinsicImpureFunction_t &x) {
+        src = "";
+        out += func_name;
+        if (x.n_args > 0) visit_expr(*x.m_args[0]);
+        out += "(" + src;
+        for (size_t i = 1; i < x.n_args; i++) {
+            out += ", ";
+            visit_expr(*x.m_args[i]);
+            out += src;
+        }
+        out += ")";
+        src = out;
+    }
+
+    void visit_IntrinsicImpureFunction(const ASR::IntrinsicImpureFunction_t &x) {
+        std::string out;
+        std::string intrinsic_func_name = ASRUtils::get_impure_intrinsic_name(static_cast<int64_t>(x.m_impure_intrinsic_id));
+        visit_IntrinsicImpureFunction_helper(out, intrinsic_func_name, x);
+    }
 
     void visit_StructConstructor(const ASR::StructConstructor_t &x) {
         std::string r = indent;
@@ -1480,28 +1762,28 @@ public:
     // void visit_UnionConstructor(const ASR::UnionConstructor_t &x) {}
 
     void visit_ImpliedDoLoop(const ASR::ImpliedDoLoop_t &x) {
-        std::string r = "("; 
+        std::string r = "(";
         for (size_t i = 0; i < x.n_values; i++) {
             visit_expr(*x.m_values[i]);
             r += src;
-            if (i != x.n_values - 1) r += ", "; 
+            if (i != x.n_values - 1) r += ", ";
         }
         r += ", ";
-        visit_expr(*x.m_var);   
+        visit_expr(*x.m_var);
         r += src;
         r += " = ";
         visit_expr(*x.m_start);
         r += src;
         r += ", ";
-        visit_expr(*x.m_end); 
+        visit_expr(*x.m_end);
         r += src;
-        if (x.m_increment) {   
+        if (x.m_increment) {
             r += ", ";
             visit_expr(*x.m_increment);
             r += src;
             }
-        r += ")"; 
-        src = r;  
+        r += ")";
+        src = r;
         return;
     }
 
@@ -1868,7 +2150,21 @@ public:
         src = r;
     }
 
-    // void visit_BitCast(const ASR::BitCast_t &x) {}
+    void visit_BitCast(const ASR::BitCast_t &x) {
+        std::string r = "transfer(";
+        visit_expr(*x.m_source);
+        r += src;
+        r += ", ";
+        visit_expr(*x.m_mold);
+        r += src;
+        if (x.m_size) {
+            r += ", ";
+            visit_expr(*x.m_size);
+            r += src;
+        }
+        r += ")";
+        src = r;
+    }
 
     void visit_StructInstanceMember(const ASR::StructInstanceMember_t &x) {
         std::string r;
@@ -2013,7 +2309,10 @@ public:
 
     // void visit_SizeOfType(const ASR::SizeOfType_t &x) {}
 
-    // void visit_PointerNullConstant(const ASR::PointerNullConstant_t &x) {}
+    void visit_PointerNullConstant(const ASR::PointerNullConstant_t &x) {
+        (void)x; // suppress unused warning
+        src = "null()";
+    }
 
     // void visit_PointerAssociated(const ASR::PointerAssociated_t &x) {}
 
@@ -2067,6 +2366,12 @@ public:
         src = r;
     }
 
+    /******************************* Ttype ********************************/
+    void visit_StructType(const ASR::StructType_t &/*x*/) {
+        std::string r = indent;
+        // r += ASRUtils::symbol_name(x.m_derived_type);
+        src = r;
+    }
 };
 
 Result<std::string> asr_to_fortran(ASR::TranslationUnit_t &asr,
