@@ -102,6 +102,16 @@ std::string LLVMModule::str()
     return LLVMEvaluator::module_to_string(*m_m);
 }
 
+llvm::Function *LLVMModule::get_function(const std::string &fn_name) {
+    llvm::Module *m = m_m.get();
+    return m->getFunction(fn_name);
+}
+
+llvm::GlobalVariable *LLVMModule::get_global(const std::string &global_name) {
+    llvm::Module *m = m_m.get();
+    return m->getNamedGlobal(global_name);
+}
+
 std::string LLVMModule::get_return_type(const std::string &fn_name)
 {
     llvm::Module *m = m_m.get();
@@ -262,7 +272,11 @@ std::unique_ptr<llvm::Module> LLVMEvaluator::parse_module(const std::string &sou
     if (v) {
         throw LCompilersException("parse_module(): module failed verification.");
     };
+#if LLVM_VERSION_MAJOR >= 21
+    module->setTargetTriple(llvm::Triple(target_triple));
+#else
     module->setTargetTriple(target_triple);
+#endif
     module->setDataLayout(jit->getDataLayout());
     return module;
 }
@@ -287,7 +301,11 @@ void LLVMEvaluator::add_module(const std::string &source) {
 void LLVMEvaluator::add_module(std::unique_ptr<llvm::Module> mod) {
     // These are already set in parse_module(), but we set it here again for
     // cases when the Module was constructed directly, not via parse_module().
+#if LLVM_VERSION_MAJOR >= 21
+    mod->setTargetTriple(llvm::Triple(target_triple));
+#else
     mod->setTargetTriple(target_triple);
+#endif
     mod->setDataLayout(jit->getDataLayout());
     llvm::Error err = jit->addModule(std::move(mod), context);
     if (err) {
@@ -369,7 +387,11 @@ void LLVMEvaluator::save_asm_file(llvm::Module &m, const std::string &filename)
 }
 
 void LLVMEvaluator::save_object_file(llvm::Module &m, const std::string &filename) {
+#if LLVM_VERSION_MAJOR >= 21
+    m.setTargetTriple(llvm::Triple(target_triple));
+#else
     m.setTargetTriple(target_triple);
+#endif
     m.setDataLayout(TM->createDataLayout());
 
     llvm::legacy::PassManager pass;
@@ -397,7 +419,11 @@ void LLVMEvaluator::create_empty_object_file(const std::string &filename) {
 }
 
 void LLVMEvaluator::opt(llvm::Module &m) {
+#if LLVM_VERSION_MAJOR >= 21
+    m.setTargetTriple(llvm::Triple(target_triple));
+#else
     m.setTargetTriple(target_triple);
+#endif
     m.setDataLayout(TM->createDataLayout());
 
 #if LLVM_VERSION_MAJOR >= 17
@@ -409,11 +435,11 @@ void LLVMEvaluator::opt(llvm::Module &m) {
     PB.registerModuleAnalyses(MAM);
     PB.registerCGSCCAnalyses(CGAM);
     PB.registerFunctionAnalyses(FAM);
-    PB.registerLoopAnalyses(LAM);   
-    PB.crossRegisterProxies(LAM, FAM, CGAM, MAM);   
+    PB.registerLoopAnalyses(LAM);
+    PB.crossRegisterProxies(LAM, FAM, CGAM, MAM);
     llvm::ModulePassManager MPM = PB.buildPerModuleDefaultPipeline(llvm::OptimizationLevel::O3);
     MPM.run(m, MAM);
-    
+
 #else
     llvm::legacy::PassManager mpm;
     mpm.add(new llvm::TargetLibraryInfoWrapperPass(TM->getTargetTriple()));
@@ -465,6 +491,10 @@ llvm::LLVMContext &LLVMEvaluator::get_context()
     return *context;
 }
 
+const llvm::DataLayout &LLVMEvaluator::get_jit_data_layout() {
+    return jit->getDataLayout();
+}
+
 void LLVMEvaluator::print_targets()
 {
     llvm::InitializeNativeTarget();
