@@ -24,6 +24,8 @@ namespace LCompilers {
 
         llvm::Value* lfortran_malloc(llvm::LLVMContext &context, llvm::Module &module,
                 llvm::IRBuilder<> &builder, llvm::Value* arg_size);
+        llvm::Value* lfortran_realloc(llvm::LLVMContext &context, llvm::Module &module,
+                llvm::IRBuilder<> &builder, llvm::Value* ptr, llvm::Value* arg_size);
 
         /*
         * This function checks whether the
@@ -116,6 +118,7 @@ namespace LCompilers {
                 */
                 virtual
                 llvm::Type* get_array_type(
+                    ASR::expr_t* expr,
                     ASR::ttype_t* m_type_,
                     llvm::Type* el_type,
                     bool get_pointer=false) = 0;
@@ -134,12 +137,12 @@ namespace LCompilers {
                 */
                 virtual
                 void fill_array_details(
-                    llvm::Value* arr, llvm::Type* llvm_data_type, int n_dims,
+                    llvm::Type* arr_ty, llvm::Value* arr, llvm::Type* llvm_data_type, int n_dims,
                     std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims,
                     llvm::Module* module, bool reserve_data_memory=true) = 0;
 
                 virtual
-                void fill_array_details(
+                void fill_array_details(ASR::expr_t* src_expr, ASR::expr_t* dest_expr,
                     llvm::Value* source, llvm::Value* destination,
                     ASR::ttype_t* source_array_type, ASR::ttype_t* dest_array_type, llvm::Module* module, bool ignore_data) = 0;
 
@@ -149,28 +152,35 @@ namespace LCompilers {
                 */
                 virtual
                 void fill_malloc_array_details(
-                    llvm::Value* arr, llvm::Type *arr_type, llvm::Type* llvm_data_type, int n_dims,
-                    std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims,
+                    llvm::Value* arr, llvm::Type *arr_type, llvm::Type* llvm_data_type, ASR::ttype_t* asr_type, int n_dims,
+                    std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims, llvm::Value* string_len,
                     llvm::Module* module, bool realloc=false) = 0;
 
                 virtual
-                void fill_dimension_descriptor(
-                    llvm::Value* arr, int n_dims, llvm::Module* module,ASR::ttype_t* type) = 0;
+                void fill_dimension_descriptor(llvm::Type* type,llvm::Value* arr, int n_dims) = 0;
 
                 virtual
                 void reset_array_details(
-                    llvm::Value* arr, llvm::Value* source_arr, int n_dims) = 0;
+                    llvm::Type* type, llvm::Value* arr, llvm::Value* source_arr, int n_dims) = 0;
+
+                virtual
+                void reset_array_details(
+                    llvm::Type* type, llvm::Value* arr, llvm::Value* source_arr,
+                    llvm::Value** lbs, llvm::Value** lengths,
+                    int n_dims) = 0;
 
                 virtual
                 void fill_descriptor_for_array_section(
-                    llvm::Value* value_desc, llvm::Type* value_el_type, llvm::Value* target,
+                    llvm::Value* value_desc, llvm::Type* value_el_type, ASR::ttype_t* value_type,
+                    llvm::Value* target, ASR::ttype_t* target_type, ASR::expr_t* target_expr,
                     llvm::Value** lbs, llvm::Value** ubs,
                     llvm::Value** ds, llvm::Value** non_sliced_indices,
                     int value_rank, int target_rank) = 0;
 
                 virtual
                 void fill_descriptor_for_array_section_data_only(
-                    llvm::Value* value_desc, llvm::Type* value_el_type, llvm::Value* target,
+                    llvm::Value* value_desc, llvm::Type* value_el_type, ASR::ttype_t* value_type,
+                    llvm::Value* target, ASR::ttype_t* target_type, ASR::expr_t* target_expr,
                     llvm::Value** lbs, llvm::Value** ubs,
                     llvm::Value** ds, llvm::Value** non_sliced_indices,
                     llvm::Value** llvm_diminfo, int value_rank, int target_rank) = 0;
@@ -190,13 +200,22 @@ namespace LCompilers {
                 virtual
                 llvm::Value* get_pointer_to_data(llvm::Value* arr) = 0;
 
+                /*
+                * Returns pointer to data in the input
+                * array descriptor according to the rules
+                * implemented by current class.
+                * Uses ASR type to get the corresponding LLVM type
+                */
+                virtual
+                llvm::Value* get_pointer_to_data(ASR::expr_t* arr_expr, ASR::ttype_t* arr_type, llvm::Value* arr, llvm::Module* module) = 0;
+
                 /*
                 * Returns offset in the input
                 * array descriptor according to the rules
                 * implemented by current class).
                 */
                 virtual
-                llvm::Value* get_offset(llvm::Value* dim_des, bool load=true) = 0;
+                llvm::Value* get_offset(llvm::Type* type, llvm::Value* dim_des, bool load=true) = 0;
 
                 /*
                 * Returns lower bound in the input
@@ -236,10 +255,10 @@ namespace LCompilers {
                     bool load=true) = 0;
 
                 virtual
-                llvm::Value* get_rank(llvm::Value* arr, bool get_pointer=false) = 0;
+                llvm::Value* get_rank(llvm::Type* type, llvm::Value* arr, bool get_pointer=false) = 0;
 
                 virtual
-                void set_rank(llvm::Value* arr, llvm::Value* rank) = 0;
+                void set_rank(llvm::Type* type, llvm::Value* arr, llvm::Value* rank) = 0;
 
                 /*
                 * Returns pointer to dimension descriptor array
@@ -249,8 +268,6 @@ namespace LCompilers {
                 virtual
                 llvm::Value* get_pointer_to_dimension_descriptor_array(llvm::Type *type, llvm::Value* arr, bool load=true) = 0;
 
-                virtual
-                llvm::Value* get_pointer_to_dimension_descriptor_array(llvm::Value* arr, bool load=true) = 0;
 
                 /*
                 * Returns pointer to the dimension descriptor
@@ -269,34 +286,41 @@ namespace LCompilers {
                 virtual
                 llvm::Value* get_single_element(llvm::Type *type, llvm::Value* array,
                     std::vector<llvm::Value*>& m_args, int n_args,
+                    ASR::ttype_t* asr_type, ASR::expr_t* expr,
                     bool data_only=false, bool is_fixed_size=false,
                     llvm::Value** llvm_diminfo=nullptr,
-                    bool polymorphic=false, llvm::Type* polymorphic_type=nullptr, bool is_unbounded_pointer_to_data = false) = 0;
+                    bool polymorphic=false, llvm::Type* polymorphic_type=nullptr,
+                    bool is_unbounded_pointer_to_data = false, bool check_for_bounds = false, std::string array_name = "") = 0;
 
                 virtual
-                llvm::Value* get_is_allocated_flag(llvm::Value* array, llvm::Type* llvm_data_type) = 0;
+                llvm::Value* get_is_allocated_flag(llvm::Value* array, llvm::Type* llvm_data_type, ASR::expr_t* array_exp) = 0;
 
                 virtual
                 void reset_is_allocated_flag(llvm::Value* array, llvm::Type* llvm_data_type) = 0;
 
 
                 virtual
-                llvm::Value* reshape(llvm::Value* array, llvm::Type* llvm_data_type,
-                                     llvm::Value* shape, ASR::ttype_t* asr_shape_type,
+                llvm::Value* reshape(llvm::Type* arr_type, llvm::Value* array, llvm::Type* llvm_data_type,
+                                     llvm::Type* shape_type, llvm::Value* shape, ASR::ttype_t* asr_shape_type,
                                      llvm::Module* module) = 0;
 
                 virtual
-                void copy_array(llvm::Value* src, llvm::Value* dest,
+                void copy_array(llvm::Type* src_ty, llvm::Value* src, llvm::Type* dest_ty, llvm::Value* dest,
                                 llvm::Module* module, ASR::ttype_t* asr_data_type,
                                 bool reserve_memory) = 0;
 
+                virtual
+                void copy_array_move_allocation(llvm::Type* src_ty, llvm::Value* src, llvm::Type* dest_ty, llvm::Value* dest,
+                                llvm::Module* module, ASR::expr_t* array_exp, ASR::ttype_t* asr_data_type) = 0;
+
                 virtual
                 void copy_array_data_only(llvm::Value* src, llvm::Value* dest,
                                           llvm::Module* module, llvm::Type* llvm_data_type,
+                                          ASR::ttype_t* arr_type,
                                           llvm::Value* num_elements) = 0;
 
                 virtual
-                llvm::Value* get_array_size(llvm::Value* array, llvm::Value* dim,
+                llvm::Value* get_array_size(llvm::Type* type, llvm::Value* array, llvm::Value* dim,
                                             int output_kind, int dim_kind=4) = 0;
 
         };
@@ -317,12 +341,12 @@ namespace LCompilers {
                 std::vector<llvm::Value*>& heap_arrays;
 
                 llvm::Value* cmo_convertor_single_element(
-                    llvm::Value* arr, std::vector<llvm::Value*>& m_args,
-                    int n_args, bool check_for_bounds);
+                    llvm::Type* type, llvm::Value* arr, std::vector<llvm::Value*>& m_args,
+                    int n_args, bool check_for_bounds, std::string array_name = "");
 
                 llvm::Value* cmo_convertor_single_element_data_only(
                     llvm::Value** llvm_diminfo, std::vector<llvm::Value*>& m_args,
-                    int n_args, bool check_for_bounds, bool is_unbounded_pointer_to_data = false);
+                    int n_args, bool check_for_bounds, bool is_unbounded_pointer_to_data = false, std::string array_name = "");
 
             public:
 
@@ -349,6 +373,7 @@ namespace LCompilers {
 
                 virtual
                 llvm::Type* get_array_type(
+                    ASR::expr_t* expr,
                     ASR::ttype_t* m_type_,
                     llvm::Type* el_type,
                     bool get_pointer=false);
@@ -358,39 +383,46 @@ namespace LCompilers {
 
                 virtual
                 void fill_array_details(
-                    llvm::Value* arr, llvm::Type* llvm_data_type, int n_dims,
+                    llvm::Type* arr_ty, llvm::Value* arr, llvm::Type* llvm_data_type, int n_dims,
                     std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims,
                     llvm::Module* module, bool reserve_data_memory=true);
 
                 virtual
-                void fill_array_details(
+                void fill_array_details(ASR::expr_t* src_expr, ASR::expr_t* dest_expr,
                     llvm::Value* source, llvm::Value* destination,
                     ASR::ttype_t* source_array_type, ASR::ttype_t* dest_array_type, llvm::Module* module, bool ignore_data);
 
                 virtual
                 void fill_malloc_array_details(
-                    llvm::Value* arr, llvm::Type *arr_type, llvm::Type* llvm_data_type, int n_dims,
-                    std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims,
+                    llvm::Value* arr, llvm::Type *arr_type, llvm::Type* llvm_data_type, ASR::ttype_t* asr_type, int n_dims,
+                    std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims, llvm::Value* string_len,
                     llvm::Module* module, bool realloc=false);
 
                 virtual
-                void fill_dimension_descriptor(
-                    llvm::Value* arr, int n_dims, llvm::Module* module, ASR::ttype_t* type);
+                void fill_dimension_descriptor(llvm::Type* type, llvm::Value* arr, int n_dims);
 
                 virtual
                 void reset_array_details(
-                    llvm::Value* arr, llvm::Value* source_arr, int n_dims);
+                    llvm::Type* type, llvm::Value* arr, llvm::Value* source_arr, int n_dims);
+
+                virtual
+                void reset_array_details(
+                    llvm::Type* type, llvm::Value* arr, llvm::Value* source_arr,
+                    llvm::Value** lbs, llvm::Value** lengths,
+                    int n_dims);
 
                 virtual
                 void fill_descriptor_for_array_section(
-                    llvm::Value* value_desc, llvm::Type* value_el_type, llvm::Value* target,
+                    llvm::Value* value_desc, llvm::Type* value_el_type, ASR::ttype_t* value_type,
+                    llvm::Value* target, ASR::ttype_t* target_type, ASR::expr_t* target_expr,
                     llvm::Value** lbs, llvm::Value** ubs,
                     llvm::Value** ds, llvm::Value** non_sliced_indices,
                     int value_rank, int target_rank);
 
                 virtual
                 void fill_descriptor_for_array_section_data_only(
-                    llvm::Value* value_desc, llvm::Type* value_el_type, llvm::Value* target,
+                    llvm::Value* value_desc, llvm::Type* value_el_type, ASR::ttype_t* value_type,
+                    llvm::Value* target, ASR::ttype_t* target_type, ASR::expr_t* target_expr,
                     llvm::Value** lbs, llvm::Value** ubs,
                     llvm::Value** ds, llvm::Value** non_sliced_indices,
                     llvm::Value** llvm_diminfo, int value_rank, int target_rank);
@@ -400,15 +432,22 @@ namespace LCompilers {
 
                 virtual
                 llvm::Value* get_pointer_to_data(llvm::Value* arr);
+                
+                /*
+                 * Return pointer to data in array descriptor,
+                 * Used arr_type to get the corresponding llvm::Type (LLVM 17+).
+                */
+                virtual
+                llvm::Value* get_pointer_to_data(ASR::expr_t* arr_expr, ASR::ttype_t* arr_type, llvm::Value* arr, llvm::Module* module);
 
                 virtual
-                llvm::Value* get_rank(llvm::Value* arr, bool get_pointer=false);
+                llvm::Value* get_rank(llvm::Type* type, llvm::Value* arr, bool get_pointer=false);
 
                 virtual
-                void set_rank(llvm::Value* arr, llvm::Value* rank);
+                void set_rank(llvm::Type* type, llvm::Value* arr, llvm::Value* rank);
 
                 virtual
-                llvm::Value* get_offset(llvm::Value* dim_des, bool load=true);
+                llvm::Value* get_offset(llvm::Type* type, llvm::Value* dim_des, bool load=true);
 
                 virtual
                 llvm::Value* get_lower_bound(llvm::Value* dim_des, bool load=true);
@@ -427,9 +466,6 @@ namespace LCompilers {
                 virtual
                 llvm::Value* get_pointer_to_dimension_descriptor_array(llvm::Type* type, llvm::Value* arr, bool load=true);
 
-                virtual
-                llvm::Value* get_pointer_to_dimension_descriptor_array(llvm::Value* arr, bool load=true);
-
                 virtual
                 llvm::Value* get_pointer_to_dimension_descriptor(llvm::Value* dim_des_arr,
                     llvm::Value* dim);
@@ -440,33 +476,40 @@ namespace LCompilers {
                 virtual
                 llvm::Value* get_single_element(llvm::Type *type, llvm::Value* array,
                     std::vector<llvm::Value*>& m_args, int n_args,
+                    ASR::ttype_t* asr_type, ASR::expr_t* expr,
                     bool data_only=false, bool is_fixed_size=false,
                     llvm::Value** llvm_diminfo=nullptr,
-                    bool polymorphic=false, llvm::Type* polymorphic_type=nullptr, bool is_unbounded_pointer_to_data = false);
+                    bool polymorphic=false, llvm::Type* polymorphic_type=nullptr,
+                    bool is_unbounded_pointer_to_data = false, bool check_for_bounds = false, std::string array_name = "");
 
                 virtual
-                llvm::Value* get_is_allocated_flag(llvm::Value* array, llvm::Type* llvm_data_type);
+                llvm::Value* get_is_allocated_flag(llvm::Value* array, llvm::Type* llvm_data_type, ASR::expr_t* array_exp);
 
                 virtual
                 void reset_is_allocated_flag(llvm::Value* array, llvm::Type* llvm_data_type);
 
                 virtual
-                llvm::Value* reshape(llvm::Value* array, llvm::Type* llvm_data_type,
-                                     llvm::Value* shape, ASR::ttype_t* asr_shape_type,
+                llvm::Value* reshape(llvm::Type* arr_type, llvm::Value* array, llvm::Type* llvm_data_type,
+                                     llvm::Type* shape_type, llvm::Value* shape, ASR::ttype_t* asr_shape_type,
                                      llvm::Module* module);
 
                 virtual
-                void copy_array(llvm::Value* src, llvm::Value* dest,
+                void copy_array(llvm::Type* src_ty, llvm::Value* src, llvm::Type* dest_ty, llvm::Value* dest,
                                 llvm::Module* module, ASR::ttype_t* asr_data_type,
                                 bool reserve_memory);
 
+                virtual
+                void copy_array_move_allocation(llvm::Type* src_ty, llvm::Value* src, llvm::Type* dest_ty, llvm::Value* dest,
+                                llvm::Module* module, ASR::expr_t* array_exp, ASR::ttype_t* asr_data_type);
+
                 virtual
                 void copy_array_data_only(llvm::Value* src, llvm::Value* dest,
                                           llvm::Module* module, llvm::Type* llvm_data_type,
+                                          ASR::ttype_t* arr_type,
                                           llvm::Value* num_elements);
 
                 virtual
-                llvm::Value* get_array_size(llvm::Value* array, llvm::Value* dim,
+                llvm::Value* get_array_size(llvm::Type* type, llvm::Value* array, llvm::Value* dim,
                                             int output_kind, int dim_kind=4);
 
         };
