@@ -27,6 +27,8 @@ log.setLevel(level)
 TESTER_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__)))
 LIBASR_DIR = os.path.dirname(TESTER_DIR)
 SRC_DIR = os.path.dirname(LIBASR_DIR)
+if "lpython" in SRC_DIR:
+    SRC_DIR = os.path.join(os.path.dirname(os.path.dirname(SRC_DIR)), "src")
 ROOT_DIR = os.path.dirname(SRC_DIR)
 
 no_color = False
@@ -256,6 +258,7 @@ def get_error_diff(reference_file, output_file, full_err_str) -> str:
 
 
 def do_update_reference(jo, jr, do):
+    os.makedirs(os.path.dirname(jr), exist_ok=True)
     shutil.copyfile(jo, jr)
     for f in ["outfile", "stdout", "stderr"]:
         if do[f]:
@@ -425,6 +428,16 @@ def tester_main(compiler, single_test, is_lcompilers_executable_installed=False)
     global no_color
     no_color = args.no_color
 
+    # Remove all old test references while updating
+    if update_reference:
+        log.debug("REMOVE: old test references")
+        cmd = "rm -rf ./tests/reference/*"
+        log.debug(f"+ {cmd}")
+        process = subprocess.run(cmd, shell=True)
+        if process.returncode != 0:
+            print("Removing Old test references failed!")
+            exit(1)
+
     # So that the tests find the `lcompiler` executable
     if not is_lcompilers_executable_installed:
         os.environ["PATH"] = os.path.join(SRC_DIR, "bin") \
