@@ -62,18 +62,6 @@ intrinsic_funcs_args = {
             "ret_type_arg_idx": 1
         },
     ],
-    "Mvbits": [
-        {
-            "args": [("int", "int", "int", "int", "int")],
-            "ret_type_arg_idx": 3
-        },
-    ],
-    "MoveAlloc": [
-        {
-            "args": [("any", "any")],
-            "ret_type_arg_idx": 0
-        },
-    ],
     "Leadz": [
         {
             "args": [("int",)],
@@ -196,12 +184,6 @@ intrinsic_funcs_args = {
             "return": "int32"
         },
     ],
-    "Repeat": [
-        {
-            "args": [("char", "int")],
-            "ret_type_arg_idx": 0
-        }
-    ],
     "StringContainsSet": [
         {
             "args": [("char", "char", "bool", "int")],
@@ -263,12 +245,6 @@ intrinsic_funcs_args = {
             "return": "int32"
         },
     ],
-    "IsContiguous": [
-        {
-            "args": [("any",)],
-            "return": "logical"
-        },
-    ],
     "OutOfRange": [
         {
             "args": [("int", "real", "bool"), ("real", "real", "bool"), ("int", "int", "bool"), ("real", "int", "bool")],
@@ -329,7 +305,8 @@ intrinsic_funcs_args = {
     "Idnint": [
         {
             "args": [("real",)],
-            "return": "int32"
+            "return": "int32",
+            "kind_validation": [{"first":{0: 8}}]
         }
     ],
     "Anint": [
@@ -512,12 +489,14 @@ intrinsic_funcs_args = {
     "Ifix": [
         {
             "args": [("real",)],
-            "return": "int32"
+            "return": "int32",
+            "kind_validation": [{"first":{0: 4}}]
         }
     ],
     "Idint": [
         {
             "args": [("real",)],
+            "kind_validation": [{"first":{0: 8}}],
             "return": "int32"
         }
     ],
@@ -690,7 +669,7 @@ intrinsic_funcs_args = {
     "NewLine": [
         {
             "args": [("char",)],
-            "return": "character(-1)"
+            "return": "character(1)"
         }
     ],
     "Range": [
@@ -735,9 +714,16 @@ intrinsic_funcs_args = {
             "ret_type_arg_idx": 0
         }
     ],
+    "Loc": [
+        {
+            "args": [("any",)],
+            "return": "int32"
+        }
+    ],
     "Dprod": [
         {
             "args": [("real", "real")],
+            "kind_validation": [{"first":{0: 4}}, {"second":{1: 4}}],
             "return": "real64"
         }
     ],
@@ -784,7 +770,10 @@ intrinsic_funcs_args = {
         {
             "args": [("char",)],
             "return": "int32",
-            "kind_arg": True
+            "kind_arg": True,
+            "char_len_validation": [
+                {1: {0: {"min": 1, "max": 1}}}
+            ]
         },
     ],
     "Char": [
@@ -854,9 +843,9 @@ intrinsic_funcs_args = {
         {
             "args": [("int",), ("real",), ("complex",)],
             "return": "real32",
-            "kind_arg": True, 
+            "kind_arg": True,
             "real_32_except_complex": True
-        },   
+        },
     ],
     "Int": [
         {
@@ -875,7 +864,7 @@ intrinsic_funcs_args = {
     "StringTrim": [
         {
             "args": [("char",)],
-            "ret_type_arg_idx": 0
+            "return" : "allocatable_deferred_string()"
         }
     ],
 }
@@ -891,6 +880,7 @@ compile_time_only_fn = [
     "Rank",
     "Tiny",
     "Huge",
+    "Loc",
     "BitSize",
     "NewLine",
     "Kind",
@@ -1023,6 +1013,34 @@ def add_create_func_arg_type_src(func_name):
             src += 4 * indent + f'append_error(diag, "Kind of all the arguments of {func_name} must be the same", loc);\n'
             src += 4 * indent + f'return nullptr;\n'
             src += 3 * indent + '}\n'
+        kind_validation_info = arg_info.get("kind_validation", [])
+        if kind_validation_info != []:
+            src += 3 * indent + "int kind = 0;\n"
+        for validation_item in kind_validation_info:
+            for arg_name, arg_spec in validation_item.items():
+                arg_pos = list(arg_spec.keys())[0]
+                required_kind = list(arg_spec.values())[0]
+                src += 3 * indent + f"kind = ASRUtils::extract_kind_from_ttype_t(expr_type(args[{arg_pos}]));\n"
+                src += 3 * indent + f"if(kind != {required_kind}) " + "{\n"
+                src += 4 * indent + f'append_error(diag, "{arg_name} argument of `{func_name.lower()}` must have kind equal to {required_kind}", loc);\n'
+                src += 4 * indent + f'return nullptr;\n'
+                src += 3 * indent + '}\n'
+        char_len_validation_info = arg_info.get("char_len_validation", [])
+        for validation_item in char_len_validation_info:
+            for arg_name, arg_spec in validation_item.items():
+                arg_pos = list(arg_spec.keys())[0]
+                constraint = list(arg_spec.values())[0]
+                src += 3 * indent + f'if (ASR::is_a<ASR::String_t>(*arg_type{arg_pos})) {{\n'
+                src += 4 * indent + f'int64_t len = ASRUtils::get_fixed_string_len(arg_type{arg_pos});\n'
+                if constraint["min"] == constraint["max"]:
+                    src += 4 * indent + f'if (len != -1 && len != {constraint["min"]}) {{\n'
+                    src += 5 * indent + f'append_error(diag, "Argument {arg_name} to {func_name} must have length {constraint["min"]}", loc);\n'
+                else:
+                    src += 4 * indent + f'if (len != -1 && (len < {constraint["min"]} || len > {constraint["max"]})) {{\n'
+                    src += 5 * indent + f'append_error(diag, "Argument {arg_name} to {func_name} must have length between {constraint["min"]} and {constraint["max"]}", loc);\n'
+                src += 5 * indent + 'return nullptr;\n'
+                src += 4 * indent + '}\n'
+                src += 3 * indent + '}\n'
         src += 2 * indent + "}\n"
     src += 2 * indent + "else {\n"
     src += 3 * indent + f'append_error(diag, "Unexpected number of args, {func_name} takes {no_of_args_msg} arguments, found " + std::to_string(args.size()), loc);\n'
@@ -1051,11 +1069,7 @@ def add_create_func_return_src(func_name):
             src += indent * 2 + f"set_kind_to_ttype_t(type_,upper_kind);\n"
         else:
             src += indent * 2 + "ASR::ttype_t* type_ = nullptr;\n"
-            src += indent * 2 + "if (use_experimental_simplifier) {\n"
-            src += indent * 3 +     f"type_ = expr_duplicator.duplicate_ttype(ASRUtils::extract_type(expr_type(args[{ret_type_arg_idx}])));\n"
-            src += indent * 2 + "} else {\n"
-            src += indent * 3 +     f"type_ = expr_duplicator.duplicate_ttype(expr_type(args[{ret_type_arg_idx}]));\n"
-            src += indent * 2 + "}\n"
+            src += indent * 2 + f"type_ = expr_duplicator.duplicate_ttype(ASRUtils::extract_type(expr_type(args[{ret_type_arg_idx}])));\n"
         ret_type = "type_"
     kind_arg = arg_infos[0].get("kind_arg", False)
     src += indent * 2 + f"ASR::ttype_t *return_type = {ret_type};\n"
@@ -1092,27 +1106,17 @@ def add_create_func_return_src(func_name):
             "ASRUtils::expr_type(m_args[0]), m_args[0], return_type, m_value);\n"
 
     else:
-        src += indent * 2 + "if (use_experimental_simplifier) {\n"
-        src += indent * 3 +     f"for( size_t i = 0; i < {no_of_args}; i++ ) " + "{\n"
-        src += indent * 4 +         "ASR::ttype_t* type = ASRUtils::expr_type(args[i]);\n"
-        src += indent * 4 +         "if (ASRUtils::is_array(type)) {\n"
-        src += indent * 5 +             "ASR::dimension_t* m_dims = nullptr;\n"
-        src += indent * 5 +             "size_t n_dims = ASRUtils::extract_dimensions_from_ttype(type, m_dims);\n"
-        src += indent * 5 +             "return_type = ASRUtils::make_Array_t_util(al, type->base.loc, "
+        src += indent * 2 +     f"for( size_t i = 0; i < {no_of_args}; i++ ) " + "{\n"
+        src += indent * 3 +         "ASR::ttype_t* type = ASRUtils::expr_type(args[i]);\n"
+        src += indent * 3 +         "if (ASRUtils::is_array(type)) {\n"
+        src += indent * 4 +             "ASR::dimension_t* m_dims = nullptr;\n"
+        src += indent * 4 +             "size_t n_dims = ASRUtils::extract_dimensions_from_ttype(type, m_dims);\n"
+        src += indent * 4 +             "return_type = ASRUtils::make_Array_t_util(al, type->base.loc, "
         src +=                              "return_type, m_dims, n_dims, ASR::abiType::Source, false, "
         src +=                              "ASR::array_physical_typeType::DescriptorArray);\n"
-        src += indent * 5 +             "break;\n"
-        src += indent * 4 +         "}\n"
-        src += indent * 3 +     "}\n"
-        src += indent * 2 + "}\n"
-        if ret_type_val:
-            src += indent * 2 + "else {\n"
-            src += indent * 3 +     "ASR::ttype_t* type = ASRUtils::expr_type(args[0]);\n"
-            src += indent * 3 +     "if (ASR::is_a<ASR::Array_t>(*type)) {\n"
-            src += indent * 4 +         "ASR::Array_t* e = ASR::down_cast<ASR::Array_t>(type);\n"
-            src += indent * 4 +         f"return_type = TYPE(ASR::make_Array_t(al, type->base.loc,  return_type, e->m_dims, e->n_dims, ASR::array_physical_typeType::FixedSizeArray));\n"
-            src += indent * 3 +     "}\n"
-            src += indent * 2 + "}\n"
+        src += indent * 4 +             "break;\n"
+        src += indent * 3 +         "}\n"
+        src += indent * 2 +     "}\n"
 
         src += indent * 2 + "if (all_args_evaluated(m_args)) {\n"
         src += indent * 3 +     f"Vec<ASR::expr_t*> args_values; args_values.reserve(al, {no_of_args});\n"
