@@ -99,8 +99,8 @@ public:
             LCOMPILERS_ASSERT(flip_sign_variable);
             ASR::expr_t* flip_sign_result = PassUtils::get_flipsign(flip_sign_signal_variable,
                                             flip_sign_variable, al, unit, x.base.base.loc, pass_options);
-            pass_result.push_back(al, ASRUtils::STMT(ASR::make_Assignment_t(al, x.base.base.loc,
-                    flip_sign_variable, flip_sign_result, nullptr)));
+            pass_result.push_back(al, ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, x.base.base.loc,
+                    flip_sign_variable, flip_sign_result, nullptr, false, false)));
         }
     }
 
