@@ -291,8 +291,19 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
             is_result_var_fixed_size = ASRUtils::is_fixed_size_array(ASRUtils::expr_type(result_var));
         }
         ASR::ttype_t* result_type_ = nullptr;
+        ASR::expr_t* non_const_len_expr = nullptr;
         bool is_allocatable = false;
         ASR::expr_t* array_constructor = get_ArrayConstructor_size(x, is_allocatable);
+
+        // Case: `keywords = [character(len=ii) :: value]`
+        // we need to allocate at runtime at string length (Here `ii`) is runtime
+        if (ASRUtils::is_character(*x->m_type)) {
+            ASR::String_t* string_type = ASR::down_cast<ASR::String_t>(ASRUtils::extract_type(x->m_type));
+            if (!ASRUtils::is_value_constant(string_type->m_len)) {
+                non_const_len_expr = string_type->m_len;
+                is_allocatable = true;
+            }
+        }
         Vec<ASR::dimension_t> dims;
         dims.reserve(al, 1);
         ASR::dimension_t dim;
@@ -312,21 +323,28 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                 result_type_ = ASRUtils::TYPE(ASR::make_Allocatable_t(al, x->m_type->base.loc,
                     ASRUtils::type_get_past_allocatable(
                         ASRUtils::duplicate_type_with_empty_dims(al, x->m_type))));
+                if (non_const_len_expr) {
+                    // If non_const_len_expr present we have to make Variable
+                    // with deffered length and then we will allocate its length
+                    ASR::down_cast<ASR::String_t>(ASRUtils::extract_type(result_type_))->m_len = nullptr;
+                    ASR::down_cast<ASR::String_t>(ASRUtils::extract_type(result_type_))->m_len_kind = ASR::string_length_kindType::DeferredLength;
+                }
             } else {
                 result_type_ = ASRUtils::duplicate_type(al,
                     ASRUtils::type_get_past_allocatable(x->m_type), &dims);
             }
         }
         result_var = PassUtils::create_var(result_counter, "_array_constructor_",
-                        loc, result_type_, al, current_scope);
+                        loc, result_type_, al, current_scope, x->m_args[0] ? x->m_args[0] : nullptr);
         result_counter += 1;
         *current_expr = result_var;
 
         Vec<ASR::alloc_arg_t> alloc_args;
         alloc_args.reserve(al, 1);
         ASR::alloc_arg_t arg;
-        arg.m_len_expr = nullptr;
+        arg.m_len_expr = non_const_len_expr;
         arg.m_type = nullptr;
+        arg.m_sym_subclass = nullptr;
         arg.m_dims = dims.p;
         arg.n_dims = dims.size();
         if( is_allocatable ) {
@@ -421,6 +439,7 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
         ASR::alloc_arg_t arg;
         arg.m_len_expr = nullptr;
         arg.m_type = nullptr;
+        arg.m_sym_subclass = nullptr;
         arg.m_dims = dims.p;
         arg.n_dims = dims.size();
         if( is_allocatable ) {
@@ -641,15 +660,20 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
                     args.reserve(al, 1);
                     args.push_back(al, x->m_values[i]);
                     ASR::expr_t* fmt_val = ASRUtils::EXPR(ASR::make_StringFormat_t(al,x->base.base.loc, nullptr,
-                        args.p, 1,ASR::string_format_kindType::FormatFortran,
-                        ASRUtils::TYPE(ASR::make_String_t(al,x->base.base.loc,-1,-1,nullptr, ASR::string_physical_typeType::PointerString)), nullptr));
+                        args.p, 1, ASR::string_format_kindType::FormatFortran,
+                        ASRUtils::TYPE(ASR::make_Allocatable_t(al, x->base.base.loc, 
+                            ASRUtils::TYPE(ASR::make_String_t(al,x->base.base.loc, 1,
+                                nullptr,
+                                ASR::string_length_kindType::DeferredLength,
+                                ASR::string_physical_typeType::DescriptorString)))),
+                        nullptr));
                     print_values.push_back(al, fmt_val);
                     if ( print ) {
                         stmt = ASRUtils::STMT(ASRUtils::make_print_t_util(al, x->m_values[i]->base.loc, print_values.p, print_values.size()));
                     } else {
                         // this will be file_write
                         LCOMPILERS_ASSERT(file_write);
-                        stmt = ASRUtils::STMT(ASR::make_FileWrite_t(al, x->m_values[i]->base.loc, 0, m_unit, nullptr, nullptr, nullptr, print_values.p, print_values.size(), nullptr, nullptr, nullptr));
+                        stmt = ASRUtils::STMT(ASR::make_FileWrite_t(al, x->m_values[i]->base.loc, 0, m_unit, nullptr, nullptr, nullptr, print_values.p, print_values.size(), nullptr, nullptr, nullptr, true));
                     }
                     do_loop_body.push_back(al, stmt);
                 }
@@ -669,7 +693,7 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
                 integer :: i
                 print *, [(i, i=1, 10)]
             */
-            if(ASR::is_a<ASR::String_t>(*ASRUtils::expr_type(x.m_text))){
+            if(ASR::is_a<ASR::String_t>(*ASRUtils::extract_type(ASRUtils::expr_type(x.m_text)))){
                 ASR::Print_t* print_stmt = const_cast<ASR::Print_t*>(&x);
                 ASR::expr_t** current_expr_copy_9 = current_expr;
                 current_expr = const_cast<ASR::expr_t**>(&(print_stmt->m_text));
@@ -811,7 +835,7 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
 void pass_replace_implied_do_loops(Allocator &al,
     ASR::TranslationUnit_t &unit,
     const LCompilers::PassOptions& pass_options) {
-    ArrayConstantVisitor v(al, pass_options.realloc_lhs);
+    ArrayConstantVisitor v(al, pass_options.realloc_lhs_arrays);
     v.visit_TranslationUnit(unit);
     PassUtils::UpdateDependenciesVisitor u(al);
     u.visit_TranslationUnit(unit);
