@@ -5,6 +5,7 @@
 #include <libasr/asr_verify.h>
 #include <libasr/pass/pass_utils.h>
 #include <libasr/pass/array_by_data.h>
+#include <libasr/asr_builder.h>
 
 #include <vector>
 #include <utility>
@@ -129,9 +130,11 @@ class PassArrayByDataProcedureVisitor : public PassUtils::PassVisitor<PassArrayB
                 }
                 if( std::find(indices.begin(), indices.end(), i) != indices.end() ) {
                     if( arg_func ) {
-                        suffix += "_" + ASRUtils::type_to_str_fortran(arg_func->m_function_signature);
+                        suffix += "_" + ASRUtils::type_to_str_fortran_expr(arg_func->m_function_signature, 
+                            ASRUtils::get_expr_from_sym(al, &arg_func->base));
                     } else {
-                        suffix += "_" + ASRUtils::type_to_str_fortran(arg->m_type);
+                        suffix += "_" + ASRUtils::type_to_str_fortran_symbol(arg->m_type, 
+                            arg->m_type_declaration);
                     }
                     suffix += "_" + std::to_string(i);
                 }
@@ -194,21 +197,26 @@ class PassArrayByDataProcedureVisitor : public PassUtils::PassVisitor<PassArrayB
                     int n_dims = ASRUtils::extract_dimensions_from_ttype(arg->m_type, dims);
                     Vec<ASR::expr_t*> dim_variables;
                     std::string arg_name = std::string(arg->m_name);
-                    PassUtils::create_vars(dim_variables, 2 * n_dims, arg->base.base.loc, al,
+                    PassUtils::create_vars(dim_variables, n_dims, arg->base.base.loc, al,
                                             x->m_symtab, arg_name, ASR::intentType::In, arg->m_presence);
                     Vec<ASR::dimension_t> new_dims;
                     new_dims.reserve(al, n_dims);
+                    ASRUtils::ASRBuilder builder(al, arg->base.base.loc);
                     for( int j = 0, k = 0; j < n_dims; j++ ) {
                         ASR::dimension_t new_dim;
                         new_dim.loc = dims[j].loc;
-                        new_dim.m_start = dim_variables[k];
-                        new_dim.m_length = dim_variables[k + 1];
+                        if (dims[j].m_start) {
+                            new_dim.m_start = dims[j].m_start;
+                        } else {
+                            new_dim.m_start = builder.i32(1);
+                        }
+                        new_dim.m_length = dim_variables[k];
                         new_dims.push_back(al, new_dim);
-                        k += 2;
+                        k += 1;
                     }
                     ASR::ttype_t* new_type = ASRUtils::duplicate_type(al, arg->m_type, &new_dims);
                     arg->m_type = new_type;
-                    for( int k = 0; k < 2 * n_dims; k++ ) {
+                    for( int k = 0; k < n_dims; k++ ) {
                         new_args.push_back(al, dim_variables[k]);
                     }
                 }
@@ -343,7 +351,7 @@ class EditProcedureReplacer: public ASR::BaseExprReplacer<EditProcedureReplacer>
         create `calcfc_internal_real____0` as replacement of `calcfc_internal` and add it as a symbol
         in scope of `cobyla`. Now, we visit `cobyla_real____0` and replace `calcfc_internal` with
         `calcfc_internal_real____0`
-        
+
         This means both symbols `cobyla` and `cobyla_real____0` have `calcfc_internal_real____0` as a symbol
         but eventually `cobyla` is removed from scope.
 
@@ -362,7 +370,7 @@ class EditProcedureReplacer: public ASR::BaseExprReplacer<EditProcedureReplacer>
         if( v.proc2newproc.find(ext_sym) != v.proc2newproc.end() ) {
             ASR::symbol_t* new_sym = v.proc2newproc[ext_sym].first;
             ASR::asr_t* new_sym_parent = ASRUtils::symbol_parent_symtab(new_sym)->asr_owner;
-            if ( ASR::is_a<ASR::symbol_t>(*new_sym_parent) && 
+            if ( ASR::is_a<ASR::symbol_t>(*new_sym_parent) &&
                  current_scope->get_counter() != ASRUtils::symbol_parent_symtab(new_sym)->get_counter() ) {
                 ASR::symbol_t* resolved_parent_sym = resolve_new_proc(ASR::down_cast<ASR::symbol_t>(new_sym_parent));
                 if ( resolved_parent_sym != nullptr ) {
@@ -395,7 +403,7 @@ class EditProcedureReplacer: public ASR::BaseExprReplacer<EditProcedureReplacer>
             }
             return ;
         }
-        
+
         edit_symbol_pointer(v)
     }
 
@@ -521,7 +529,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
             if( v.proc2newproc.find(ext_sym) != v.proc2newproc.end() ) {
                 ASR::symbol_t* new_sym = v.proc2newproc[ext_sym].first;
                 ASR::asr_t* new_sym_parent = ASRUtils::symbol_parent_symtab(new_sym)->asr_owner;
-                if ( ASR::is_a<ASR::symbol_t>(*new_sym_parent) && 
+                if ( ASR::is_a<ASR::symbol_t>(*new_sym_parent) &&
                         current_scope->get_counter() != ASRUtils::symbol_parent_symtab(new_sym)->get_counter() ) {
                     ASR::symbol_t* resolved_parent_sym = resolve_new_proc(ASR::down_cast<ASR::symbol_t>(new_sym_parent));
                     if ( resolved_parent_sym != nullptr ) {
@@ -573,10 +581,6 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
             ASR::dimension_t* compile_time_dims = nullptr;
             int n_dims = ASRUtils::extract_dimensions_from_ttype(array_type, compile_time_dims);
             for( int i = 0; i < n_dims; i++ ) {
-                ASR::expr_t* start = compile_time_dims[i].m_start;
-                if( start == nullptr ) {
-                    start = PassUtils::get_bound(array, i + 1, "lbound", al);
-                }
                 ASR::expr_t* length = compile_time_dims[i].m_length;
                 if( length == nullptr ) {
                     if (ASRUtils::is_allocatable(array_type)) {
@@ -585,7 +589,16 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                         length = PassUtils::get_bound(array, i + 1, "ubound", al);
                     }
                 }
-                dims.push_back(al, start);
+                if ( ASRUtils::is_integer(*ASRUtils::expr_type(length)) ) {
+                    ASR::Integer_t* int_type = ASR::down_cast<ASR::Integer_t>(ASRUtils::type_get_past_array(
+                                                ASRUtils::type_get_past_allocatable_pointer(ASRUtils::expr_type(length))));
+                    if (int_type->m_kind != 4) {
+                        // we know by default dimensions are construced as `4`
+                        length = ASRUtils::EXPR(ASRUtils::make_Cast_t_value(al, length->base.loc, length, ASR::cast_kindType::IntegerToInteger,
+                            ASRUtils::TYPE(ASR::make_Integer_t(al, length->base.loc, 4))));
+                    }
+
+                }
                 dims.push_back(al, length);
             }
         }
@@ -598,7 +611,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                     new_args.push_back(al, orig_args[i]);
                     continue;
                 } else if (std::find(indices.begin(), indices.end(), i) == indices.end()) {
-                    ASR::expr_t* expr = orig_args[i].m_value;    
+                    ASR::expr_t* expr = orig_args[i].m_value;
                     if (ASR::is_a<ASR::Var_t>(*expr)) {
                         ASR::Var_t* var = ASR::down_cast<ASR::Var_t>(expr);
                         ASR::symbol_t* sym = var->m_v;
@@ -617,11 +630,11 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                 if( ASRUtils::is_array(orig_arg_type) ) {
                     ASR::Array_t* array_t = ASR::down_cast<ASR::Array_t>(
                         ASRUtils::type_get_past_allocatable(ASRUtils::type_get_past_pointer(orig_arg_type)));
-                    if( array_t->m_physical_type != ASR::array_physical_typeType::PointerToDataArray ) {
+                    if( array_t->m_physical_type != ASR::array_physical_typeType::PointerArray ) {
                         ASR::expr_t* physical_cast = ASRUtils::EXPR(ASRUtils::make_ArrayPhysicalCast_t_util(
                             al, orig_arg_i->base.loc, orig_arg_i, array_t->m_physical_type,
-                            ASR::array_physical_typeType::PointerToDataArray, ASRUtils::duplicate_type(al, orig_arg_type,
-                            nullptr, ASR::array_physical_typeType::PointerToDataArray, true), nullptr));
+                            ASR::array_physical_typeType::PointerArray, ASRUtils::duplicate_type(al, orig_arg_type,
+                            nullptr, ASR::array_physical_typeType::PointerArray, true), nullptr));
                         ASR::call_arg_t physical_cast_arg;
                         physical_cast_arg.loc = orig_arg_i->base.loc;
                         physical_cast_arg.m_value = physical_cast;
@@ -684,7 +697,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
 
                 if ( v.proc2newproc.find(subrout_sym) != v.proc2newproc.end()) {
                     ASR::symbol_t* new_x_name = nullptr;
-                    if (is_external && (v.proc2newproc.find(x.m_name) != v.proc2newproc.end())) { 
+                    if (is_external && (v.proc2newproc.find(x.m_name) != v.proc2newproc.end())) {
                         new_x_name = v.proc2newproc[x.m_name].first;
                     } else {
                         new_x_name = resolve_new_proc(x.m_name);
@@ -707,7 +720,16 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                     return;
                 }
             }
-            if( !can_edit_call(x.m_args, x.n_args) ) {
+
+            // Currently we don't transform class procedure functions
+            if (ASR::is_a<ASR::StructMethodDeclaration_t>(*subrout_sym)) {
+                subrout_sym = ASRUtils::symbol_get_past_external(
+                    ASRUtils::symbol_get_past_StructMethodDeclaration(subrout_sym));
+                update_args_for_pass_arr_by_data_funcs_passed_as_callback(x);
+                not_to_be_erased.insert(subrout_sym);
+                return;
+            }
+            if( !can_edit_call(x.m_args, x.n_args) && !ASRUtils::get_FunctionType(subrout_sym)->m_module ) {
                 not_to_be_erased.insert(subrout_sym);
                 return ;
             }
@@ -804,7 +826,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
             }
             current_scope = current_scope_copy;
         }
-        
+
         void visit_Program(const ASR::Program_t &x) {
             ASR::Program_t& xx = const_cast<ASR::Program_t&>(x);
             SymbolTable* current_scope_copy = current_scope;
@@ -855,7 +877,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
             }
             // Case: procedure(cb) :: call_back (Here call_back is variable of type cb which is a function)
             ASR::symbol_t* type_dec = x.m_type_declaration;
-            if (v.proc2newproc.find(ASRUtils::symbol_get_past_external(type_dec)) != v.proc2newproc.end() && 
+            if (v.proc2newproc.find(ASRUtils::symbol_get_past_external(type_dec)) != v.proc2newproc.end() &&
                     v.proc2newproc.find(type_dec) == v.proc2newproc.end()){
                 ASR::symbol_t* new_func = resolve_new_proc(ASRUtils::symbol_get_past_external(type_dec));
                 ASR::ExternalSymbol_t* x_sym_ext = ASR::down_cast<ASR::ExternalSymbol_t>(type_dec);
@@ -869,7 +891,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                 v.proc2newproc[type_dec] = {new_func_sym_, {}};
                 x_sym_ext->m_parent_symtab->add_symbol(new_func_sym_name, new_func_sym_);
             }
-            if(type_dec && v.proc2newproc.find(type_dec) != v.proc2newproc.end() && 
+            if(type_dec && v.proc2newproc.find(type_dec) != v.proc2newproc.end() &&
                 ASR::is_a<ASR::Function_t>(*ASRUtils::symbol_get_past_external(type_dec))){
                 ASR::symbol_t* new_sym = resolve_new_proc(type_dec);
                 if ( new_sym == nullptr ) {
@@ -890,12 +912,12 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                 }
                 std::string new_sym_name = x.m_parent_symtab->get_unique_name(x.m_name);
                 ASR::symbol_t* new_func_sym_ = ASR::down_cast<ASR::symbol_t>(
-                    ASRUtils::make_Variable_t_util(v.al, x.base.base.loc, x.m_parent_symtab, s2c(v.al, new_sym_name), 
-                        x.m_dependencies, x.n_dependencies, x.m_intent, 
-                        sym_val, m_val, x.m_storage, subrout->m_function_signature, 
+                    ASRUtils::make_Variable_t_util(v.al, x.base.base.loc, x.m_parent_symtab, s2c(v.al, new_sym_name),
+                        x.m_dependencies, x.n_dependencies, x.m_intent,
+                        sym_val, m_val, x.m_storage, subrout->m_function_signature,
                         new_sym, x.m_abi, x.m_access, x.m_presence, x.m_value_attr));
                 v.proc2newproc[(ASR::symbol_t *) &x] = {new_func_sym_, {}};
-                x.m_parent_symtab->add_symbol(new_sym_name, new_func_sym_); 
+                x.m_parent_symtab->add_symbol(new_sym_name, new_func_sym_);
                 not_to_be_erased.insert(new_func_sym_);
             }
             ASR::ASRPassBaseWalkVisitor<EditProcedureCallsVisitor>::visit_Variable(x);
@@ -925,8 +947,8 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                 sim.m_type = ASRUtils::symbol_type(new_func_sym_);
             }
         }
- 
-        void visit_Struct(const ASR::Struct_t &x) { 
+
+        void visit_Struct(const ASR::Struct_t &x) {
             //just to update names of changed symbols of Struct
             ASR::ASRPassBaseWalkVisitor<EditProcedureCallsVisitor>::visit_Struct(x);
             ASR::Struct_t& ss = const_cast<ASR::Struct_t&>(x);
@@ -937,7 +959,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                 }
             }
         }
-      
+
         void visit_Var(const ASR::Var_t &x) {
             if (v.proc2newproc.find(x.m_v) != v.proc2newproc.end()){
                 ASR::symbol_t* new_sym = v.proc2newproc[x.m_v].first;
@@ -1003,7 +1025,7 @@ class RemoveArrayByDescriptorProceduresVisitor : public PassUtils::PassVisitor<R
                     SymbolTable* current_scope_copy = current_scope;
                     visit_symbol(*item.second);
                     current_scope = current_scope_copy;
-                    if (func_type->n_arg_types >= 1 && ASR::is_a<ASR::ClassType_t>(*func_type->m_arg_types[0])) {
+                    if (func_type->n_arg_types >= 1 && ASRUtils::is_class_type(func_type->m_arg_types[0])) {
                         continue;
                     }
                 }
@@ -1020,7 +1042,8 @@ class RemoveArrayByDescriptorProceduresVisitor : public PassUtils::PassVisitor<R
                     ASR::is_a<ASR::Program_t>(*item.second) ||
                     ASR::is_a<ASR::Function_t>(*item.second) ||
                     ASR::is_a<ASR::Struct_t>(*item.second) ||
-                    ASR::is_a<ASR::Block_t>(*item.second)) {
+                    ASR::is_a<ASR::Block_t>(*item.second) ||
+                    ASR::is_a<ASR::AssociateBlock_t>(*item.second)) {
                     SymbolTable* current_scope_copy = current_scope;
                     visit_symbol(*item.second);
                     current_scope = current_scope_copy;
@@ -1060,10 +1083,13 @@ class RemoveArrayByDescriptorProceduresVisitor : public PassUtils::PassVisitor<R
             visit_Unit(x);
         }
 
+        void visit_AssociateBlock(const ASR::AssociateBlock_t& x) {
+            visit_Unit(x);
+        }
 };
 
 void pass_array_by_data(Allocator &al, ASR::TranslationUnit_t &unit,
-                        const LCompilers::PassOptions& /*pass_options*/) {
+                        const LCompilers::PassOptions& pass_options) {
     PassArrayByDataProcedureVisitor v(al);
     v.visit_TranslationUnit(unit);
     EditProcedureVisitor e(v);
@@ -1072,7 +1098,23 @@ void pass_array_by_data(Allocator &al, ASR::TranslationUnit_t &unit,
     EditProcedureCallsVisitor u(al, v, not_to_be_erased);
     u.visit_TranslationUnit(unit);
     RemoveArrayByDescriptorProceduresVisitor x(al, v, not_to_be_erased);
-    x.visit_TranslationUnit(unit);
+    if ( !pass_options.skip_removal_of_unused_procedures_in_pass_array_by_data ) {
+        /*
+            If separate compilation is enabled using `--separate-compilation`, then we don't
+            drop the original ( unused ) procedures. This is for the module procedures where when
+            loaded from other file transformation may or maynot take place and then while linking
+            it shows missing symbol. There can be multiple reasons:
+
+            1. Module procedure is transformed in current file but not in the other file. -- this approach fixes it.
+                a. Function `get_midpoints(A, B)` where A(:) and B(:) are arrays, get transformed in current file, but
+                   in other file, it got called as `print *, get_midpoints(A(1:3), b(1:3))`, LFortran treats these as
+                   pointers and hence doesn't transform it.
+            2. Module procedure is not transformed in current file but is done in other file -- very less prone to happen.
+                a. For the functions where reshape is used over Function arguments, it is not transformed in current file but
+                   in other file it is transformed as these routines are marked as external ( meaning n_body = 0 ).
+        */
+        x.visit_TranslationUnit(unit);
+    }
     PassUtils::UpdateDependenciesVisitor y(al);
     y.visit_TranslationUnit(unit);
 }
