@@ -93,18 +93,15 @@ namespace LCompilers {
             return get_rank(x) > 0;
         }
 
-         #define fix_struct_type_scope() array_ref_type = ASRUtils::type_get_past_array( \
-                ASRUtils::type_get_past_pointer( \
-                    ASRUtils::type_get_past_allocatable(array_ref_type))); \
-            if( current_scope && ASR::is_a<ASR::StructType_t>(*array_ref_type) ) { \
-                ASR::StructType_t* struct_t = ASR::down_cast<ASR::StructType_t>(array_ref_type); \
-                if( current_scope->get_counter() != ASRUtils::symbol_parent_symtab( \
-                        struct_t->m_derived_type)->get_counter() ) { \
-                    ASR::symbol_t* m_derived_type = current_scope->resolve_symbol( \
-                        ASRUtils::symbol_name(struct_t->m_derived_type)); \
-                    ASR::ttype_t* struct_type = ASRUtils::TYPE(ASR::make_StructType_t(al, \
-                        struct_t->base.base.loc, m_derived_type)); \
-                    array_ref_type = struct_type; \
+        #define fix_struct_type_scope() if ( arr_expr != nullptr ) { \
+                ASR::symbol_t* m_derived_type = ASRUtils::symbol_get_past_external(ASRUtils::get_struct_sym_from_struct_expr(arr_expr)); \
+                if (m_derived_type != nullptr && current_scope != nullptr && \
+                    current_scope->get_counter() != ASRUtils::symbol_parent_symtab(m_derived_type)->get_counter()) { \
+                    ASR::symbol_t* new_derived_type = current_scope->get_symbol( \
+                        ASRUtils::symbol_name(m_derived_type)); \
+                    if (new_derived_type) { \
+                        ASRUtils::set_struct_sym_to_struct_expr(arr_expr, new_derived_type); \
+                    } \
                 } \
             } \
 
@@ -121,7 +118,18 @@ namespace LCompilers {
             args.push_back(al, ai);
             ASR::ttype_t* array_ref_type = ASRUtils::duplicate_type_without_dims(
                 al, ASRUtils::expr_type(arr_expr), arr_expr->base.loc);
-            fix_struct_type_scope()
+            // fix_struct_type_scope()
+            if ( arr_expr != nullptr ) {
+                ASR::symbol_t* m_derived_type = ASRUtils::symbol_get_past_external(ASRUtils::get_struct_sym_from_struct_expr(arr_expr));
+                if (m_derived_type != nullptr && current_scope != nullptr &&
+                    current_scope->get_counter() != ASRUtils::symbol_parent_symtab(m_derived_type)->get_counter()) {
+                    ASR::symbol_t* new_derived_type = current_scope->get_symbol(
+                        ASRUtils::symbol_name(m_derived_type));
+                    if (new_derived_type) {
+                        ASRUtils::set_struct_sym_to_struct_expr(arr_expr, new_derived_type);
+                    }
+                }
+            }
             ASR::expr_t* array_ref = ASRUtils::EXPR(ASRUtils::make_ArrayItem_t_util(al,
                                         arr_expr->base.loc, arr_expr,
                                         args.p, args.size(),
@@ -230,10 +238,11 @@ namespace LCompilers {
             Vec<ASR::dimension_t> empty_dims;
             empty_dims.reserve(al, 1);
             ASR::ttype_t* array_ref_type = array_section->m_type;
+            ASR::expr_t* arr_expr = array_section->m_v;
             array_ref_type = ASRUtils::duplicate_type_without_dims(al, array_ref_type, loc);
             fix_struct_type_scope()
             ASR::expr_t* array_ref = ASRUtils::EXPR(ASRUtils::make_ArrayItem_t_util(al,
-                                        loc, array_section->m_v,
+                                        loc, arr_expr,
                                         args.p, args.size(),
                                         array_ref_type,
                                         ASR::arraystorageType::RowMajor, nullptr));
@@ -246,7 +255,7 @@ namespace LCompilers {
         }
 
         ASR::expr_t* create_array_ref(ASR::symbol_t* arr, Vec<ASR::expr_t*>& idx_vars, Allocator& al,
-            const Location& loc, ASR::ttype_t* _type, SymbolTable* current_scope, bool perform_cast,
+            const Location& loc, ASR::ttype_t* _type, SymbolTable* /*current_scope*/, bool perform_cast,
             ASR::cast_kindType cast_kind, ASR::ttype_t* casted_type) {
             Vec<ASR::array_index_t> args;
             args.reserve(al, 1);
@@ -259,7 +268,9 @@ namespace LCompilers {
                 args.push_back(al, ai);
             }
             ASR::ttype_t* array_ref_type = ASRUtils::duplicate_type_without_dims(al, _type, loc);
-            fix_struct_type_scope()
+            // TODO: in future pass expr from where _type is derived, so that we can
+            // fix the struct
+            // fix_struct_type_scope()
             ASR::expr_t* arr_var = ASRUtils::EXPR(ASR::make_Var_t(al, loc, arr));
             ASR::expr_t* array_ref = ASRUtils::EXPR(ASRUtils::make_ArrayItem_t_util(al, loc, arr_var,
                                         args.p, args.size(),
@@ -315,7 +326,7 @@ namespace LCompilers {
             return false;
         }
         ASR::expr_t* create_var(int counter, std::string suffix, const Location& loc,
-                            ASR::ttype_t* var_type, Allocator& al, SymbolTable*& current_scope) {
+                            ASR::ttype_t* var_type, Allocator& al, SymbolTable*& current_scope, ASR::expr_t* var) {
             ASR::dimension_t* m_dims = nullptr;
             int ndims = 0;
             PassUtils::get_dim_rank(var_type, m_dims, ndims);
@@ -332,12 +343,17 @@ namespace LCompilers {
             if( current_scope->get_symbol(str_name) == nullptr ||
                 !ASRUtils::check_equal_type(
                     ASRUtils::symbol_type(current_scope->get_symbol(str_name)),
-                    var_type, true
+                    var_type, ASRUtils::get_expr_from_sym(
+                        al, current_scope->get_symbol(str_name)), var, true
                 ) ) {
+                ASR::symbol_t* type_decl = nullptr;
+                if ( var != nullptr ) {
+                    type_decl = ASRUtils::get_struct_sym_from_struct_expr(var);
+                }
                 str_name = current_scope->get_unique_name(str_name);
                 ASR::asr_t* idx_sym = ASRUtils::make_Variable_t_util(al, loc, current_scope, s2c(al, str_name), nullptr, 0,
                                                         ASR::intentType::Local, nullptr, nullptr, ASR::storage_typeType::Default,
-                                                        var_type, nullptr, ASR::abiType::Source, ASR::accessType::Public,
+                                                        var_type, type_decl, ASR::abiType::Source, ASR::accessType::Public,
                                                         ASR::presenceType::Required, false);
                 current_scope->add_symbol(str_name, ASR::down_cast<ASR::symbol_t>(idx_sym));
                 idx_var = ASRUtils::EXPR(ASR::make_Var_t(al, loc, ASR::down_cast<ASR::symbol_t>(idx_sym)));
@@ -377,7 +393,7 @@ namespace LCompilers {
                     ASR::symbol_t* idx_sym = current_scope->get_symbol(idx_var_name);
                     if( ASR::is_a<ASR::Variable_t>(*idx_sym) ) {
                         ASR::Variable_t* idx_var = ASR::down_cast<ASR::Variable_t>(idx_sym);
-                        if( !(ASRUtils::check_equal_type(idx_var->m_type, int32_type) &&
+                        if( !(ASRUtils::check_equal_type(idx_var->m_type, int32_type, nullptr, nullptr) &&
                               idx_var->m_symbolic_value == nullptr) ) {
                             idx_var_name = current_scope->get_unique_name(idx_var_name, false);
                         }
@@ -422,7 +438,7 @@ namespace LCompilers {
                     ASR::symbol_t* idx_sym = current_scope->get_symbol(idx_var_name);
                     if( ASR::is_a<ASR::Variable_t>(*idx_sym) ) {
                         ASR::Variable_t* idx_var = ASR::down_cast<ASR::Variable_t>(idx_sym);
-                        if( !(ASRUtils::check_equal_type(idx_var->m_type, int32_type) &&
+                        if( !(ASRUtils::check_equal_type(idx_var->m_type, int32_type, nullptr, nullptr) &&
                               idx_var->m_symbolic_value == nullptr) ) {
                             idx_var_name = current_scope->get_unique_name(idx_var_name, false);
                         }
@@ -466,7 +482,7 @@ namespace LCompilers {
                     ASR::symbol_t* idx_sym = current_scope->get_symbol(idx_var_name);
                     if( ASR::is_a<ASR::Variable_t>(*idx_sym) ) {
                         ASR::Variable_t* idx_var = ASR::down_cast<ASR::Variable_t>(idx_sym);
-                        if( !(ASRUtils::check_equal_type(idx_var->m_type, int32_type) &&
+                        if( !(ASRUtils::check_equal_type(idx_var->m_type, int32_type, nullptr, nullptr) &&
                               idx_var->m_symbolic_value == nullptr) ) {
                             idx_var_name = current_scope->get_unique_name(idx_var_name, false);
                         }
@@ -504,10 +520,11 @@ namespace LCompilers {
             // We tell `load_module` not to run verify, since the ASR might
             // not be in valid state. We run verify at the end of this pass
             // anyway, so verify will be run no matter what.
+            std::set<std::string> empty_set;
             LCompilers::LocationManager loc_manager;
             ASR::Module_t *m = ASRUtils::load_module(al, current_scope,
                                             module_name, loc, true,
-                                            pass_options, false,
+                                            empty_set, pass_options, false,
                                             [&](const std::string &msg, const Location &) { throw LCompilersException(msg); },
                                             loc_manager
                                             );
@@ -544,10 +561,11 @@ namespace LCompilers {
             // We tell `load_module` not to run verify, since the ASR might
             // not be in valid state. We run verify at the end of this pass
             // anyway, so verify will be run no matter what.
+            std::set<std::string> empty_set;
             LCompilers::LocationManager loc_manager;
             ASR::Module_t *m = ASRUtils::load_module(al, current_scope,
                                             module_name, loc, true,
-                                            pass_options, false,
+                                            empty_set, pass_options, false,
                                             [&](const std::string &msg, const Location &) { throw LCompilersException(msg); },
                                             loc_manager);
 
@@ -567,6 +585,19 @@ namespace LCompilers {
             return v;
         }
 
+        ASR::asr_t* make_Assignment_t_util(Allocator &al, const Location &a_loc,
+            ASR::expr_t* a_target, ASR::expr_t* a_value,
+            ASR::stmt_t* a_overloaded, bool a_realloc_lhs) {
+            bool is_allocatable = ASRUtils::is_allocatable(a_target);
+            if ( ASR::is_a<ASR::StructInstanceMember_t>(*a_target) ) {
+                ASR::StructInstanceMember_t* a_target_struct = ASR::down_cast<ASR::StructInstanceMember_t>(a_target);
+                is_allocatable |= ASRUtils::is_allocatable(a_target_struct->m_v);
+            }
+            a_realloc_lhs = a_realloc_lhs && is_allocatable;
+            return ASR::make_Assignment_t(al, a_loc, a_target, a_value,
+                a_overloaded, a_realloc_lhs, false);
+        }
+
         ASR::stmt_t* create_do_loop_helper_count(Allocator &al, const Location &loc, std::vector<ASR::expr_t*> do_loop_variables, ASR::expr_t* mask, ASR::expr_t* res, int curr_idx) {
             ASRUtils::ASRBuilder b(al, loc);
 
@@ -692,6 +723,14 @@ namespace LCompilers {
                 for (size_t i = 0; i < do_loop_variables.size(); i++) {
                     vars.push_back(do_loop_variables[i]);
                 }
+                if (ASRUtils::extract_n_dims_from_ttype(ASRUtils::expr_type(mask)) == 0) {
+                    return b.DoLoop(do_loop_variables[curr_idx - 1], LBound(array, curr_idx), UBound(array, curr_idx), {
+                        b.If(mask, {
+                            b.Assignment(b.ArrayItem_01(res, {idx}), b.ArrayItem_01(array, vars)),
+                            b.Assignment(idx, b.Add(idx, ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 1, ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4))))))
+                        }, {}),
+                    }, nullptr);
+                }
                 return b.DoLoop(do_loop_variables[curr_idx - 1], LBound(array, curr_idx), UBound(array, curr_idx), {
                     b.If(b.ArrayItem_01(mask, vars), {
                         b.Assignment(b.ArrayItem_01(res, {idx}), b.ArrayItem_01(array, vars)),
@@ -704,6 +743,15 @@ namespace LCompilers {
             }, nullptr);
         }
 
+        ASR::expr_t* create_array_size_pack(Allocator &al, const Location &loc, ASR::expr_t* array, int n_dims) {
+            ASRUtils::ASRBuilder b(al, loc);
+            if (n_dims == 1) {
+                return b.ArraySize(array, ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 1, ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)))), ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)));   
+            }
+            ASR::expr_t* dim = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, n_dims, ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4))));
+            return b.Mul(b.ArraySize(array, dim, ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4))), create_array_size_pack(al, loc, array, n_dims-1));
+        }
+
         ASR::stmt_t* create_do_loop_helper_unpack(Allocator &al, const Location &loc, std::vector<ASR::expr_t*> do_loop_variables, ASR::expr_t* vector, ASR::expr_t* mask, ASR::expr_t* res, ASR::expr_t* idx, int curr_idx) {
             ASRUtils::ASRBuilder b(al, loc);
             if (curr_idx == 1) {
@@ -723,6 +771,32 @@ namespace LCompilers {
             }, nullptr);
         }
 
+        ASR::stmt_t* create_do_loop_helper_cshift(Allocator &al, const Location &loc, std::vector<ASR::expr_t*> do_loop_variables,
+                    ASR::expr_t* array_var, ASR::expr_t* res_var, ASR::expr_t* array, ASR::expr_t* res, int curr_idx) {
+            ASRUtils::ASRBuilder b(al, loc);
+            if ( do_loop_variables.size() == 0 ) {
+                return b.Assignment(b.ArrayItem_01(res, {res_var}), b.ArrayItem_01(array, {array_var}));
+            }
+            if (curr_idx == (int)do_loop_variables.size() - 1) {
+                std::vector<ASR::expr_t*> array_vars;
+                array_vars.push_back(array_var);
+                for (size_t i = 0; i < do_loop_variables.size(); i++) {
+                    array_vars.push_back(do_loop_variables[i]);
+                } 
+                std::vector<ASR::expr_t*> res_vars;
+                res_vars.push_back(res_var);
+                for (size_t i = 0; i < do_loop_variables.size(); i++) {
+                    res_vars.push_back(do_loop_variables[i]);
+                }
+                return b.DoLoop(do_loop_variables[curr_idx], b.i32(1), UBound(array, curr_idx + 2), {
+                    b.Assignment(b.ArrayItem_01(res, res_vars), b.ArrayItem_01(array, array_vars)),
+                }, nullptr);
+            } 
+            return b.DoLoop(do_loop_variables[curr_idx], LBound(array, curr_idx + 2), UBound(array, curr_idx + 2), {
+                create_do_loop_helper_cshift(al, loc, do_loop_variables, array_var, res_var, array, res, curr_idx + 1)
+            }, nullptr);
+        }
+
         ASR::stmt_t* create_do_loop_helper_random_number(Allocator &al, const Location &loc, std::vector<ASR::expr_t*> do_loop_variables,
                     ASR::symbol_t* s, ASR::expr_t* arr, ASR::ttype_t* return_type, ASR::expr_t* arr_item, ASR::stmt_t* stmt, int curr_idx) {
             ASRUtils::ASRBuilder b(al, loc);
@@ -783,11 +857,22 @@ namespace LCompilers {
             ASR::ttype_t* type = ASRUtils::expr_type(left);
             ASR::ttype_t* cmp_type = ASRUtils::TYPE(ASR::make_Logical_t(al, loc, 4));
             // TODO: compute `value`:
+            int lhs_kind = ASRUtils::extract_kind_from_ttype_t(type);
+            int rhs_kind = ASRUtils::extract_kind_from_ttype_t(ASRUtils::expr_type(right));
             if (ASRUtils::is_integer(*type)) {
+                if (lhs_kind != rhs_kind) {
+                    right = ASRUtils::EXPR(ASR::make_Cast_t(al, loc, right, ASR::cast_kindType::IntegerToInteger, type, nullptr));
+                }
                 return ASRUtils::EXPR(ASR::make_IntegerCompare_t(al, loc, left, op, right, cmp_type, nullptr));
             } else if (ASRUtils::is_real(*type)) {
+                if (lhs_kind != rhs_kind) {
+                    right = ASRUtils::EXPR(ASR::make_Cast_t(al, loc, right, ASR::cast_kindType::RealToReal, type, nullptr));
+                }
                 return ASRUtils::EXPR(ASR::make_RealCompare_t(al, loc, left, op, right, cmp_type, nullptr));
             } else if (ASRUtils::is_complex(*type)) {
+                if (lhs_kind != rhs_kind) {
+                    right = ASRUtils::EXPR(ASR::make_Cast_t(al, loc, right, ASR::cast_kindType::ComplexToComplex, type, nullptr));
+                }
                 return ASRUtils::EXPR(ASR::make_ComplexCompare_t(al, loc, left, op, right, cmp_type, nullptr));
             } else if (ASRUtils::is_logical(*type)) {
                 return ASRUtils::EXPR(ASR::make_LogicalCompare_t(al, loc, left, op, right, cmp_type, nullptr));
@@ -919,32 +1004,35 @@ namespace LCompilers {
             ASR::asr_t* expr_sym = ASRUtils::make_Variable_t_util(al, expr->base.loc, current_scope, s2c(al, name), nullptr, 0,
                                                     ASR::intentType::Local, nullptr, nullptr, ASR::storage_typeType::Default,
                                                     ASRUtils::duplicate_type(al, ASRUtils::extract_type(ASRUtils::expr_type(expr))),
-                                                    nullptr, ASR::abiType::Source, ASR::accessType::Public, ASR::presenceType::Required, false);
+                                                    ASRUtils::get_struct_sym_from_struct_expr(expr), ASR::abiType::Source, ASR::accessType::Public, ASR::presenceType::Required, false);
             if( current_scope->get_symbol(name) == nullptr ) {
                 current_scope->add_symbol(name, ASR::down_cast<ASR::symbol_t>(expr_sym));
             } else {
                 throw LCompilersException("Symbol with " + name + " is already present in " + std::to_string(current_scope->counter));
             }
             ASR::expr_t* var = ASRUtils::EXPR(ASR::make_Var_t(al, expr->base.loc, ASR::down_cast<ASR::symbol_t>(expr_sym)));
-            assign_stmt = ASRUtils::STMT(ASR::make_Assignment_t(al, var->base.loc, var, expr, nullptr));
+            assign_stmt = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, var->base.loc, var, expr, nullptr, false, false));
             return var;
         }
 
         ASR::expr_t* create_auxiliary_variable(const Location& loc, std::string& name,
             Allocator& al, SymbolTable*& current_scope, ASR::ttype_t* var_type,
-            ASR::intentType var_intent, ASR::symbol_t* var_decl) {
-            ASRUtils::import_struct_t(al, loc, var_type, var_intent, current_scope);
+            ASR::intentType var_intent, ASR::symbol_t* var_decl, ASR::expr_t* var) {
+            ASRUtils::import_struct_t(al, loc, var_type, var_intent, current_scope, var);
+            if ( var_decl == nullptr && var != nullptr ) {
+                var_decl = ASRUtils::get_struct_sym_from_struct_expr(var);
+            }
             ASR::asr_t* expr_sym = ASRUtils::make_Variable_t_util(al, loc, current_scope, s2c(al, name), nullptr, 0,
                                                     var_intent, nullptr, nullptr, ASR::storage_typeType::Default,
-                                                    var_type, var_decl, ASR::abiType::Source, ASR::accessType::Public,
+                                                    var_type, var_decl,
+                                                    ASR::abiType::Source, ASR::accessType::Public,
                                                     ASR::presenceType::Required, false);
             if( current_scope->get_symbol(name) == nullptr ) {
                 current_scope->add_symbol(name, ASR::down_cast<ASR::symbol_t>(expr_sym));
             } else {
                 throw LCompilersException("Symbol with " + name + " is already present in " + std::to_string(current_scope->counter));
             }
-            ASR::expr_t* var = ASRUtils::EXPR(ASR::make_Var_t(al, loc, ASR::down_cast<ASR::symbol_t>(expr_sym)));
-            return var;
+            return ASRUtils::EXPR(ASR::make_Var_t(al, loc, ASR::down_cast<ASR::symbol_t>(expr_sym)));
         }
 
         ASR::expr_t* get_fma(ASR::expr_t* arg0, ASR::expr_t* arg1, ASR::expr_t* arg2,
@@ -984,12 +1072,12 @@ namespace LCompilers {
         }
 
         ASR::symbol_t* insert_fallback_vector_copy(Allocator& al, ASR::TranslationUnit_t& unit,
-            SymbolTable*& global_scope, std::vector<ASR::ttype_t*>& types,
+            SymbolTable*& global_scope, std::vector<ASR::expr_t*>& arrays, std::vector<ASR::ttype_t*>& types,
             std::string prefix) {
             const int num_args = 6;
             std::string vector_copy_name = prefix;
-            for( ASR::ttype_t*& type: types ) {
-                vector_copy_name += ASRUtils::type_to_str_python(type, false);
+            for( size_t i = 0; i < types.size(); i++ ) {
+                vector_copy_name += ASRUtils::type_to_str_python_expr(types[i], arrays[i]);
             }
             vector_copy_name += "@IntrinsicOptimization";
             if( global_scope->resolve_symbol(vector_copy_name) ) {
@@ -1023,8 +1111,8 @@ namespace LCompilers {
             loop_body.reserve(al, 1);
             ASR::expr_t* target = create_array_ref(arg_exprs[0], idx_vars, al);
             ASR::expr_t* value = create_array_ref(arg_exprs[1], idx_vars, al);
-            ASR::stmt_t* copy_stmt = ASRUtils::STMT(ASR::make_Assignment_t(al, target->base.loc,
-                                        target, value, nullptr));
+            ASR::stmt_t* copy_stmt = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, target->base.loc,
+                                        target, value, nullptr, false, false));
             loop_body.push_back(al, copy_stmt);
             ASR::stmt_t* fallback_loop = ASRUtils::STMT(ASR::make_DoLoop_t(al, do_loop_head.loc,
                                             nullptr, do_loop_head, loop_body.p, loop_body.size(), nullptr, 0));
@@ -1054,8 +1142,9 @@ namespace LCompilers {
             ASR::ttype_t* array1_type = ASRUtils::expr_type(array1);
             ASR::ttype_t* index_type = ASRUtils::expr_type(start);
             std::vector<ASR::ttype_t*> types = {array0_type, array1_type, index_type};
+            std::vector<ASR::expr_t*> arrays = {array0, array1, start};
             ASR::symbol_t *v = insert_fallback_vector_copy(al, unit, global_scope,
-                                                           types,
+                                                           arrays, types,
                                                            "vector_copy_");
             Vec<ASR::call_arg_t> args;
             args.reserve(al, 6);
@@ -1076,7 +1165,7 @@ namespace LCompilers {
             args.push_back(al, arg5_);
             return ASRUtils::STMT(ASRUtils::make_SubroutineCall_t_util(al, loc, v,
                                                              nullptr, args.p, args.size(),
-                                                             nullptr, nullptr, false, false));
+                                                             nullptr, nullptr, false));
         }
 
         ASR::expr_t* get_sign_from_value(ASR::expr_t* arg0, ASR::expr_t* arg1,
@@ -1195,7 +1284,7 @@ namespace LCompilers {
             ASR::ttype_t* type = ASRUtils::TYPE(ASR::make_Integer_t(al, loc, a_kind));
 
             ASR::stmt_t* decrement_stmt = ASRUtils::STMT(
-                                            ASR::make_Assignment_t(
+                                            ASRUtils::make_Assignment_t_util(
                                                 al,
                                                 loc,
                                                 target,
@@ -1203,7 +1292,7 @@ namespace LCompilers {
                                                     ASR::make_IntegerBinOp_t(
                                                     al, loc, target, ASR::binopType::Sub,
                                                     increment, type, nullptr)),
-                                                nullptr));
+                                                nullptr, false, false));
 
             for (size_t i = 0; i < loop.n_body; i++) {
                 if (ASR::is_a<ASR::Exit_t>(*loop.m_body[i])) {
@@ -1336,18 +1425,18 @@ namespace LCompilers {
                 int a_kind = ASRUtils::extract_kind_from_ttype_t(ASRUtils::expr_type(target));
                 ASR::ttype_t *type = ASRUtils::TYPE(ASR::make_Integer_t(al, loc, a_kind));
 
-                loop_init_stmt = ASRUtils::STMT(ASR::make_Assignment_t(al, loc, target,
+                loop_init_stmt = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, loc, target,
                     ASRUtils::EXPR(ASR::make_IntegerBinOp_t(al, loc, a,
-                            ASR::binopType::Sub, c, type, nullptr)), nullptr));
+                            ASR::binopType::Sub, c, type, nullptr)), nullptr, false, false));
                 if (use_loop_variable_after_loop) {
-                    stmt_add_c_after_loop = ASRUtils::STMT(ASR::make_Assignment_t(al, loc, target,
+                    stmt_add_c_after_loop = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, loc, target,
                         ASRUtils::EXPR(ASR::make_IntegerBinOp_t(al, loc, target,
-                                ASR::binopType::Add, c, type, nullptr)), nullptr));
+                                ASR::binopType::Add, c, type, nullptr)), nullptr, false, false));
                 }
 
-                inc_stmt = ASRUtils::STMT(ASR::make_Assignment_t(al, loc, target,
+                inc_stmt = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, loc, target,
                             ASRUtils::EXPR(ASR::make_IntegerBinOp_t(al, loc, target,
-                                ASR::binopType::Add, c, type, nullptr)), nullptr));
+                                ASR::binopType::Add, c, type, nullptr)), nullptr, false, false));
                 if (cond == nullptr) {
                     ASR::ttype_t *log_type = ASRUtils::TYPE(ASR::make_Logical_t(al, loc, 4));
                     ASR::expr_t* left = ASRUtils::EXPR(ASR::make_IntegerBinOp_t(al, loc, target,
@@ -1403,7 +1492,7 @@ namespace LCompilers {
                 ASR::expr_t* curr_init = ASRUtils::fetch_ArrayConstant_value(al, x, k);
                 ASR::expr_t* res = PassUtils::create_array_ref(arr_var, idx_var,
                     al, current_scope);
-                if( perform_cast && !ASRUtils::types_equal(ASRUtils::expr_type(curr_init), casted_type) ) {
+                if( perform_cast && !ASRUtils::types_equal(ASRUtils::expr_type(curr_init), casted_type, nullptr, nullptr) ) {
                     curr_init = ASRUtils::EXPR(ASR::make_Cast_t(
                         al, curr_init->base.loc, curr_init, cast_kind, casted_type, nullptr));
                 }
@@ -1453,7 +1542,7 @@ namespace LCompilers {
                 ASR::expr_t* curr_init = ASRUtils::fetch_ArrayConstant_value(al, x, k);
                 ASR::expr_t* res = PassUtils::create_array_ref(arr_var, idx_vars,
                     al, current_scope);
-                if( perform_cast && !ASRUtils::types_equal(ASRUtils::expr_type(curr_init), casted_type) ) {
+                if( perform_cast && !ASRUtils::types_equal(ASRUtils::expr_type(curr_init), casted_type, nullptr, nullptr) ) {
                     curr_init = ASRUtils::EXPR(ASR::make_Cast_t(
                         al, curr_init->base.loc, curr_init, cast_kind, casted_type, nullptr));
                 }
@@ -1504,7 +1593,7 @@ namespace LCompilers {
                         }, current_scope, result_vec);
                     } else {
                         ASR::expr_t* res = PassUtils::create_array_ref(arr_var, idx_var, al, current_scope);
-                        if( perform_cast && !ASRUtils::types_equal(ASRUtils::expr_type(curr_init), casted_type) ) {
+                        if( perform_cast && !ASRUtils::types_equal(ASRUtils::expr_type(curr_init), casted_type, nullptr, nullptr) ) {
                             curr_init = ASRUtils::EXPR(ASR::make_Cast_t(
                                 al, curr_init->base.loc, curr_init, cast_kind, casted_type, nullptr));
                         }
@@ -1609,7 +1698,7 @@ namespace LCompilers {
                             ASRUtils::type_get_past_array(
                                 ASRUtils::type_get_past_allocatable_pointer(
                                     ASRUtils::expr_type(arr_var))),
-                            dims.p, dims.size(), ASR::abiType::Source, false, 
+                            dims.p, dims.size(), ASR::abiType::Source, false,
                             ASR::array_physical_typeType::DescriptorArray,
                             false, false, !is_alloc_return_func);
                         ASR::expr_t* res = ASRUtils::EXPR(ASR::make_ArraySection_t(
