@@ -59,7 +59,11 @@ public:
         PassUtils::create_idx_vars(idx_vars, n_dims, loc, al, current_scope);
         ASR::stmt_t* doloop = nullptr;
         ASR::ttype_t *str_type_len_2 = ASRUtils::TYPE(ASR::make_String_t(
-            al, loc, 1, 0, nullptr, ASR::string_physical_typeType::PointerString));
+            al, loc, 1, 
+            ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 0,
+                ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)))),
+            ASR::string_length_kindType::ExpressionLength,
+            ASR::string_physical_typeType::DescriptorString));
         ASR::expr_t *empty_space = ASRUtils::EXPR(ASR::make_StringConstant_t(
             al, loc, s2c(al, ""), str_type_len_2));
         ASR::stmt_t* empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, loc, empty_space));
@@ -165,20 +169,24 @@ public:
                 print_args.p, print_args.size()));
         } else if (_type == ASR::stmtType::FileWrite) {
             statement = ASRUtils::STMT(ASR::make_FileWrite_t(al, loc, 0, unit,
-                nullptr, nullptr, nullptr, print_args.p, print_args.size(), separator, end, overloaded));
+                nullptr, nullptr, nullptr, print_args.p, print_args.size(), separator, end, overloaded, true));
         }
         print_body.clear();
         return statement;
     }
 
     void visit_Print(const ASR::Print_t& x) {
-        LCOMPILERS_ASSERT(ASR::is_a<ASR::String_t>(*ASRUtils::expr_type(x.m_text)));
+        LCOMPILERS_ASSERT(ASR::is_a<ASR::String_t>(*ASRUtils::extract_type(ASRUtils::expr_type(x.m_text))));
         if (ASR::is_a<ASR::StringFormat_t>(*x.m_text)) {
             std::vector<ASR::expr_t*> print_body;
             ASR::stmt_t* empty_print_endl;
             ASR::stmt_t* print_stmt;
             ASR::ttype_t *str_type_len_2 = ASRUtils::TYPE(ASR::make_String_t(
-            al, x.base.base.loc, 1, 0, nullptr, ASR::string_physical_typeType::PointerString));
+            al, x.base.base.loc, 1, 
+            ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, x.base.base.loc, 0,
+                ASRUtils::TYPE(ASR::make_Integer_t(al, x.base.base.loc, 4)))),
+            ASR::string_length_kindType::ExpressionLength,
+            ASR::string_physical_typeType::DescriptorString));
             ASR::expr_t *empty_space = ASRUtils::EXPR(ASR::make_StringConstant_t(
             al, x.base.base.loc, s2c(al, ""), str_type_len_2));
             empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, x.base.base.loc, empty_space));
@@ -216,11 +224,15 @@ public:
         PassUtils::create_idx_vars(idx_vars, n_dims, loc, al, current_scope);
         ASR::stmt_t* doloop = nullptr;
         ASR::ttype_t *str_type_len = ASRUtils::TYPE(ASR::make_String_t(
-            al, loc, 1, 0, nullptr, ASR::string_physical_typeType::PointerString));
+            al, loc, 1, 
+            ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 0,
+                ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)))),
+            ASR::string_length_kindType::ExpressionLength,
+            ASR::string_physical_typeType::DescriptorString));
         ASR::expr_t *empty_space = ASRUtils::EXPR(ASR::make_StringConstant_t(
             al, loc, s2c(al, ""), str_type_len));
         ASR::stmt_t* empty_file_write_endl = ASRUtils::STMT(ASR::make_FileWrite_t(al, loc,
-                0, unit, nullptr, nullptr, nullptr, nullptr, 0, nullptr, nullptr, nullptr));
+                0, unit, nullptr, nullptr, nullptr, nullptr, 0, nullptr, nullptr, nullptr, true));
         for( int i = n_dims - 1; i >= 0; i-- ) {
             ASR::do_loop_head_t head;
             head.m_v = idx_vars[i];
@@ -245,11 +257,11 @@ public:
                     format_args.push_back(al, string_format);
                     write_stmt = ASRUtils::STMT(ASR::make_FileWrite_t(
                         al, loc, i, unit, nullptr, nullptr, nullptr,
-                        format_args.p, format_args.size(), nullptr, empty_space, nullptr));
+                        format_args.p, format_args.size(), nullptr, empty_space, nullptr, true));
                 } else {
                     write_stmt = ASRUtils::STMT(ASR::make_FileWrite_t(
                         al, loc, i, unit, nullptr, nullptr, nullptr,
-                        print_args.p, print_args.size(), nullptr, nullptr, nullptr));
+                        print_args.p, print_args.size(), nullptr, nullptr, nullptr, true));
                 }
                 doloop_body.push_back(al, write_stmt);
             } else {
@@ -266,7 +278,7 @@ public:
         body.from_pointer_n_copy(al, write_body.data(), write_body.size());
         ASR::stmt_t* write_stmt = ASRUtils::STMT(ASR::make_FileWrite_t(
             al, x.base.base.loc, x.m_label, x.m_unit, x.m_iomsg,
-            x.m_iostat, x.m_id, body.p, body.size(), x.m_separator, x.m_end, x.m_overloaded));
+            x.m_iostat, x.m_id, body.p, body.size(), x.m_separator, x.m_end, x.m_overloaded, true));
         pass_result.push_back(al, write_stmt);
         write_body.clear();
     }
