@@ -100,8 +100,8 @@ public:
             call_args.push_back(al, call_arg);
         }
         return ASRUtils::EXPR(ASRUtils::make_FunctionCall_t_util(al, loc,
-            sym, sym, call_args.p, call_args.n, return_type, nullptr, nullptr,
-            false));
+            sym, sym, call_args.p, call_args.n, return_type, nullptr, nullptr
+            ));
     }
 
     ASR::symbol_t *create_bindc_function(const Location &loc,
@@ -118,13 +118,13 @@ public:
             for (auto &type: args_type) {
                 std::string arg_name = "x_0" + std::to_string(i); i++;
                 args.push_back(al, b.Variable(fn_symtab, arg_name, type,
-                    ASR::intentType::In, ASR::abiType::BindC, true));
+                    ASR::intentType::In, nullptr, ASR::abiType::BindC, true));
             }
             ASR::expr_t *return_var = nullptr;
             if ( return_type ) {
                 char *return_var_name = s2c(al, "_lpython_return_variable");
                 return_var = b.Variable(fn_symtab, return_var_name, return_type,
-                    ASR::intentType::ReturnVar, ASR::abiType::BindC, false);
+                    ASR::intentType::ReturnVar, nullptr, ASR::abiType::BindC, false);
             }
 
             Vec<ASR::stmt_t *> body; body.reserve(al, 1);
@@ -200,9 +200,17 @@ public:
     ASR::expr_t* basic_str(const Location& loc, ASR::expr_t *x) {
         ASR::symbol_t* basic_str_sym = create_bindc_function(loc,
             "basic_str", {ASRUtils::TYPE(ASR::make_CPtr_t(al, loc))},
-            ASRUtils::TYPE(ASR::make_String_t(al, loc, 1, -2, nullptr, ASR::string_physical_typeType::PointerString)));
+            ASRUtils::TYPE(ASR::make_String_t(al, loc, 1, 
+             ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 1, 
+                    ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)))),
+                ASR::string_length_kindType::ExpressionLength,
+                ASR::string_physical_typeType::CChar)));
         return FunctionCall(loc, basic_str_sym, {x},
-            ASRUtils::TYPE(ASR::make_String_t(al, loc, 1, -2, nullptr, ASR::string_physical_typeType::PointerString)));
+            ASRUtils::TYPE(ASR::make_String_t(al, loc, 1, 
+             ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 1, 
+                    ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)))),
+                ASR::string_length_kindType::ExpressionLength,
+                ASR::string_physical_typeType::CChar)));
     }
 
     ASR::expr_t* basic_get_type(const Location& loc, ASR::expr_t* value) {
@@ -233,7 +241,11 @@ public:
     ASR::stmt_t *symbol_set(const Location &loc, ASR::expr_t *target, ASR::expr_t *value) {
         ASR::symbol_t* symbol_set_sym = create_bindc_function(loc, "symbol_set",
             {ASRUtils::TYPE(ASR::make_CPtr_t(al, loc)), ASRUtils::TYPE(
-            ASR::make_String_t(al, loc, 1, -2, nullptr, ASR::string_physical_typeType::PointerString))});
+            ASR::make_String_t(al, loc, 1, 
+                ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 1, 
+                    ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)))),
+                ASR::string_length_kindType::ExpressionLength,
+                ASR::string_physical_typeType::CChar))});
         return SubroutineCall(loc, symbol_set_sym, {target, value});
     }
 
@@ -387,7 +399,7 @@ public:
                     ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, xx.base.base.loc, 0, type2))));
 
                 // statement 2
-                ASR::expr_t* value2 = ASRUtils::EXPR(ASR::make_PointerNullConstant_t(al, xx.base.base.loc, CPtr_type));
+                ASR::expr_t* value2 = ASRUtils::EXPR(ASR::make_PointerNullConstant_t(al, xx.base.base.loc, CPtr_type, nullptr));
 
                 // statement 3
                 ASR::expr_t* get_pointer_node = ASRUtils::EXPR(ASR::make_GetPointer_t(al, xx.base.base.loc,
@@ -396,9 +408,9 @@ public:
                     CPtr_type, nullptr));
 
                 // defining the assignment statement
-                ASR::stmt_t* stmt1 = ASRUtils::STMT(ASR::make_Assignment_t(al, xx.base.base.loc, target1, value1, nullptr));
-                ASR::stmt_t* stmt2 = ASRUtils::STMT(ASR::make_Assignment_t(al, xx.base.base.loc, target2, value2, nullptr));
-                ASR::stmt_t* stmt3 = ASRUtils::STMT(ASR::make_Assignment_t(al, xx.base.base.loc, target2, value3, nullptr));
+                ASR::stmt_t* stmt1 = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, xx.base.base.loc, target1, value1, nullptr, false, false));
+                ASR::stmt_t* stmt2 = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, xx.base.base.loc, target2, value2, nullptr, false, false));
+                ASR::stmt_t* stmt3 = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, xx.base.base.loc, target2, value3, nullptr, false, false));
                 // statement 4
                 ASR::stmt_t* stmt4 = basic_new_stack(x.base.base.loc, target2);
 
@@ -469,7 +481,7 @@ public:
                 // Statement 1
                 ASR::expr_t* args = ASRUtils::EXPR(ASR::make_Var_t(al, loc, args_sym));
                 ASR::expr_t* function_call1 = vecbasic_new(loc);
-                ASR::stmt_t* stmt1 = ASRUtils::STMT(ASR::make_Assignment_t(al, loc, args, function_call1, nullptr));
+                ASR::stmt_t* stmt1 = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, loc, args, function_call1, nullptr, false, false));
                 pass_result.push_back(al, stmt1);
 
                 // Statement 2
@@ -481,7 +493,10 @@ public:
                         x->m_args[1], ASRUtils::TYPE(ASR::make_Logical_t(al, loc, 4)), nullptr));
                 std::string error_str = "tuple index out of range";
                 ASR::ttype_t *str_type = ASRUtils::TYPE(ASR::make_String_t(al, loc,
-                        1, error_str.size(), nullptr, ASR::string_physical_typeType::PointerString));
+                    1, ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, error_str.size(),
+                        ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 8)))),
+                    ASR::string_length_kindType::ExpressionLength,
+                    ASR::string_physical_typeType::DescriptorString));
                 ASR::expr_t* error = ASRUtils::EXPR(ASR::make_StringConstant_t(al, loc, s2c(al, error_str), str_type));
                 ASR::stmt_t *stmt3 = ASRUtils::STMT(ASR::make_Assert_t(al, loc, test, error));
                 pass_result.push_back(al, stmt3);
@@ -540,7 +555,7 @@ public:
             } else if (intrinsic_func->m_type->type == ASR::ttypeType::Logical) {
                 if (is_logical_intrinsic_symbolic(x.m_value)) {
                     ASR::expr_t* function_call = process_attributes(x.base.base.loc, x.m_value);
-                    ASR::stmt_t* stmt = ASRUtils::STMT(ASR::make_Assignment_t(al, x.base.base.loc, x.m_target, function_call, nullptr));
+                    ASR::stmt_t* stmt = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, x.base.base.loc, x.m_target, function_call, nullptr, false, false));
                     pass_result.push_back(al, stmt);
                 }
             }
@@ -618,13 +633,13 @@ public:
 
                             ASR::expr_t* temp_list_const1 = ASRUtils::EXPR(ASR::make_ListConstant_t(al, x.base.base.loc, temp_list1.p,
                                             temp_list1.size(), list_type));
-                            ASR::stmt_t* stmt1 = ASRUtils::STMT(ASR::make_Assignment_t(al, x.base.base.loc, placeholder_target, temp_list_const1, nullptr));
+                            ASR::stmt_t* stmt1 = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, x.base.base.loc, placeholder_target, temp_list_const1, nullptr, false, false));
                             pass_result.push_back(al, stmt1);
 
                             // Step2: Add the empty list variable
                             ASR::expr_t* temp_list_const2 = ASRUtils::EXPR(ASR::make_ListConstant_t(al, x.base.base.loc, temp_list2.p,
                                             temp_list2.size(), list_type));
-                            ASR::stmt_t* stmt2 = ASRUtils::STMT(ASR::make_Assignment_t(al, x.base.base.loc, x.m_target, temp_list_const2, nullptr));
+                            ASR::stmt_t* stmt2 = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, x.base.base.loc, x.m_target, temp_list_const2, nullptr, false, false));
                             pass_result.push_back(al, stmt2);
 
                             // Step3: Add the list index to the function scope
@@ -636,8 +651,8 @@ public:
                                 int32_type, nullptr, ASR::abiType::Source, ASR::Public, ASR::presenceType::Required, false));
                             current_scope->add_symbol(symbolic_list_index, index_sym);
                             ASR::expr_t* index = ASRUtils::EXPR(ASR::make_Var_t(al, x.base.base.loc, index_sym));
-                            ASR::stmt_t* stmt3 = ASRUtils::STMT(ASR::make_Assignment_t(al, x.base.base.loc, index,
-                                ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, x.base.base.loc, 0, int32_type)), nullptr));
+                            ASR::stmt_t* stmt3 = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, x.base.base.loc, index,
+                                ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, x.base.base.loc, 0, int32_type)), nullptr, false, false));
                             pass_result.push_back(al, stmt3);
 
                             // Step4: Add the DoLoop for appending elements into the list
@@ -645,10 +660,10 @@ public:
                             SymbolTable* block_symtab = al.make_new<SymbolTable>(current_scope);
                             char *tmp_var_name = s2c(al, "tmp");
                             ASR::expr_t* tmp_var = b.Variable(block_symtab, tmp_var_name, CPtr_type,
-                                ASR::intentType::Local, ASR::abiType::Source, false);
+                                ASR::intentType::Local, nullptr, ASR::abiType::Source, false);
                             Vec<ASR::stmt_t*> block_body; block_body.reserve(al, 1);
-                            ASR::stmt_t* block_stmt1 = ASRUtils::STMT(ASR::make_Assignment_t(al, x.base.base.loc, tmp_var,
-                                basic_new_heap(x.base.base.loc), nullptr));
+                            ASR::stmt_t* block_stmt1 = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, x.base.base.loc, tmp_var,
+                                basic_new_heap(x.base.base.loc), nullptr, false, false));
                             block_body.push_back(al, block_stmt1);
                             ASR::stmt_t* block_stmt2 = ASRUtils::STMT(ASR::make_ListAppend_t(al, x.base.base.loc, x.m_target, tmp_var));
                             block_body.push_back(al, block_stmt2);
@@ -689,7 +704,7 @@ public:
                 } else {
                     function_call = basic_compare(x.base.base.loc, "basic_neq", s->m_left, s->m_right);
                 }
-                ASR::stmt_t* stmt = ASRUtils::STMT(ASR::make_Assignment_t(al, x.base.base.loc, x.m_target, function_call, nullptr));
+                ASR::stmt_t* stmt = ASRUtils::STMT(ASRUtils::make_Assignment_t_util(al, x.base.base.loc, x.m_target, function_call, nullptr, false, false));
                 pass_result.push_back(al, stmt);
             }
         }
@@ -728,7 +743,7 @@ public:
             } else {
                 function_call = basic_compare(xx.base.base.loc, "basic_neq", s->m_left, s->m_right);
             }
-            ASR::stmt_t* stmt = ASRUtils::STMT(ASR::make_If_t(al, xx.base.base.loc, function_call,
+            ASR::stmt_t* stmt = ASRUtils::STMT(ASR::make_If_t(al, xx.base.base.loc, nullptr, function_call,
                 xx.m_body, xx.n_body, xx.m_orelse, xx.n_orelse));
             pass_result.push_back(al, stmt);
         }
