@@ -4,8 +4,7 @@ from llanguage_server.cxx.lsp_file_generator import gensym_context
 from llanguage_server.cxx.visitors import BaseCPlusPlusLspVisitor
 from llanguage_server.lsp.datatypes import LspSpec
 from llanguage_server.lsp.utils import (lower_first, method_to_camel_case,
-                                        receive_fn, rename_enum, rename_field,
-                                        send_fn, upper_first)
+                                        receive_fn, send_fn, upper_first)
 from llanguage_server.lsp.visitors import (LspAnalysisPipeline, LspSymbol)
 
 
@@ -36,6 +35,9 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
                         'logger',
                     ]),
                 ],
+                inits=[
+                    ('logger', ['logger.having("LspLanguageServer")'])
+                ],
         ):
             self.write('// empty')
         self.newline()
@@ -51,7 +53,7 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
         ):
             with self.gen_switch('method'):
                 for request_spec in self.schema["requests"]:
-                    if request_spec["messageDirection"] == "clientToServer":
+                    if request_spec["messageDirection"] in ["clientToServer", "both"]:
                         request_method = request_spec["method"]
                         request_name = method_to_camel_case(request_method)
                         result_name = f'{request_name}Result'
@@ -64,12 +66,12 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
                                 with self.indent():
                                     self.write(f'transformer.as{request_name}Params(messageParams);')
                                 self.write(f'{result_name} result =')
-                                with self.indent(): self.write(f'{receive_fn(request_method)}(requestParams);')
+                                with self.indent(): self.write(f'{receive_fn(request_method)}(request, requestParams);')
                                 self.write(f'response.result =')
                                 with self.indent():
                                     self.write(f'transformer.{lower_first(result_name)}ToAny(result);')
                             else:
-                                self.write(f'{result_name} result = {receive_fn(request_method)}();')
+                                self.write(f'{result_name} result = {receive_fn(request_method)}(request);')
                                 self.write(f'response.result =')
                                 with self.indent():
                                     self.write(f'transformer.{lower_first(result_name)}ToAny(result);')
@@ -93,7 +95,7 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
         ):
             with self.gen_switch('method'):
                 for notification_spec in self.schema["notifications"]:
-                    if notification_spec["messageDirection"] == "clientToServer":
+                    if notification_spec["messageDirection"] in ["clientToServer", "both"]:
                         notification_method = notification_spec["method"]
                         notification_name = method_to_camel_case(notification_method)
                         notification_enum = method_to_camel_case(notification_method)
@@ -107,9 +109,9 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
                                 self.write(f'{params_spec["name"]} notificationParams =')
                                 with self.indent():
                                     self.write(f'transformer.as{notification_name}Params(messageParams);')
-                                self.write(f'{receive_fn(notification_method)}(notificationParams);')
+                                self.write(f'{receive_fn(notification_method)}(notification, notificationParams);')
                             else:
-                                self.write(f'{receive_fn(notification_method)}();')
+                                self.write(f'{receive_fn(notification_method)}(notification);')
                             self.gen_break()
                 with self.gen_default():
                     with self.gen_if('notification.method.compare(0, 2, "$/") == 0', end=''):
@@ -135,6 +137,20 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
                     'const std::string &method'
                 ]
         ):
+            self.write('RequestId requestId;')
+            with self.gen_switch('response.id.type()'):
+                with self.gen_case('ResponseIdType', 'integer'):
+                    self.gen_assign('requestId', 'response.id.integer()')
+                    self.gen_break()
+                with self.gen_case('ResponseIdType', 'string'):
+                    self.gen_assign('requestId', 'response.id.string()')
+                    self.gen_break()
+                with self.gen_default():
+                    self.gen_throw_invalid_params(
+                        '("Invalid ResponseId type: " +'
+                        ' ResponseIdTypeNames.at(response.id.type()))'
+                    )
+            self.newline()
             self.write('OutgoingRequest request;')
             with self.gen_try():
                 self.write('request = outgoingRequestByValue(method);')
@@ -143,12 +159,22 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
             self.newline()
             with self.gen_switch('request'):
                 for request_spec in self.schema["requests"]:
-                    if request_spec["messageDirection"] == "serverToClient":
+                    if request_spec["messageDirection"] in ["serverToClient", "both"]:
                         request_method = request_spec["method"]
                         request_name = method_to_camel_case(request_method)
                         result_name = f'{request_name}Result'
                         request_enum = method_to_camel_case(request_method)
                         with self.gen_case('OutgoingRequest', request_enum):
+                            self.write('auto readLock = LSP_READ_LOCK(requestMutex, "requests");')
+                            self.gen_assign('auto iter', 'requestsById.find(requestId.integer())')
+                            with self.gen_if('iter == requestsById.end()'):
+                                self.write('logger.error()')
+                                with self.indent():
+                                    self.write('<< "Cannot locate request with id=" << requestId.integer()')
+                                    self.write('<< std::endl;')
+                                self.write('return;')
+                            self.gen_assign('std::shared_ptr<RequestMessage> request', 'iter->second')
+                            self.write('readLock.unlock();')
                             result_spec = request_spec.get("result", None)
                             if result_spec is not None:
                                 with self.gen_if('!response.result.has_value()'):
@@ -166,9 +192,13 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
                                     f'{result_name} params',
                                     f'transformer.anyTo{upper_first(result_name)}(result)'
                                 )
-                                self.write(f'{receive_fn(request_method)}(params);')
+                                self.gen_call(f'{receive_fn(request_method)}', '*request', 'response', 'params')
                             else:
-                                self.write(f'{receive_fn(request_method)}()')
+                                self.gen_call(f'{receive_fn(request_method)}', 'request', 'response')
+                            self.write('auto writeLock = LSP_WRITE_LOCK(requestMutex, "requests");')
+                            self.gen_assign('iter', 'requestsById.find(requestId.integer())')
+                            with self.gen_if('iter != requestsById.end()'):
+                                self.gen_call('requestsById.erase', 'iter')
                             self.gen_break()
                 with self.gen_default():
                     self.write('invalidMethod:')
@@ -215,7 +245,9 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
         request_method = request_spec["method"]
         request_name = method_to_camel_case(request_method)
         result_name = f'{request_name}Result'
-        params = []
+        params = [
+            'const RequestMessage &/*request*/',
+        ]
         params_spec = request_spec.get("params", None)
         if params_spec is not None:
             params.append(f'{params_spec["name"]} &/*params*/')
@@ -236,7 +268,9 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
     def generate_receive_notification(self, notification_spec: LspSpec) -> None:
         notification_method = notification_spec["method"]
         receive_notification = receive_fn(notification_method)
-        params = []
+        params = [
+            'const NotificationMessage &/*notification*/',
+        ]
         params_spec = notification_spec.get("params", None)
         if params_spec is not None:
             params.append(f'{params_spec["name"]} &/*params*/')
@@ -286,7 +320,10 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
         request_method = request_spec["method"]
         receive_request = receive_fn(request_method)
         request_name = method_to_camel_case(request_method)
-        params = []
+        params = [
+            'const RequestMessage &/*request*/',
+            'const ResponseMessage &/*response*/',
+        ]
         result_spec = request_spec.get("result", None)
         if result_spec is not None:
             result_name = f'{request_name}Result'
@@ -385,6 +422,7 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
                 'const RequestMessage &request',
             ],
         ):
+            request_id_name = self.gensym_init('requestId', 'request.id.integer()')
             any_name = self.gensym_init(
                 'any',
                 'transformer.requestMessageToAny(request)'
@@ -393,6 +431,16 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
                 'message',
                 f'serializer.serialize({any_name})'
             )
+            with self.block():
+                self.write('auto writeLock = LSP_WRITE_LOCK(requestMutex, "requests");')
+                iter_name = self.gensym_init('iter', f'requestsById.find({request_id_name})')
+                with self.gen_if_eq(iter_name, 'requestsById.end()'):
+                    self.gen_call(
+                        'requestsById.emplace_hint',
+                        iter_name,
+                        request_id_name,
+                        'std::make_shared<RequestMessage>(std::move(request))'
+                    )
             self.gen_call('ls::LanguageServer::send', message_name)
         self.newline()
 
@@ -522,7 +570,6 @@ class CPlusPlusLspLanguageServerSourceGenerator(BaseCPlusPlusLspVisitor):
         print(f'Generating: {self.file_path}')
         self.generate_disclaimer()
         self.gen_include('cctype')
-        self.gen_include('cstdio')
         self.gen_include('iostream')
         self.gen_include('stdexcept')
         self.newline()
